--------------------------------------------------------------------------------
-- CERN BE-CO-HT
-- WR2RF_VME core
-- https://ohwr.org/project/vme-rf-wr-bobr
--------------------------------------------------------------------------------
--
-- unit name:   vtu_blk
--
-- description: vtu block (vtu_core + serdes + delays)
--
--------------------------------------------------------------------------------
-- Copyright (c) 2020 CERN (home.cern)
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

use work.wishbone_pkg.all;

entity vtu_blk is
    port (
      --  System clock (125Mhz or 62.5Mhz)
      clk_sys_i  : in  std_logic;
      --  System reset clock (> 50ns)
      rst_sys_i  : in  std_logic;

      --  Wishbone interface, uses clk_sys.
      wb_i       : in    t_wishbone_slave_in;
      wb_o       : out   t_wishbone_slave_out;

      -- RF clock (~200Mhz) for serdes
      clk_rf_i : in  std_logic;

      --  RF clock / 8 (for vtu)
      clk_vtu_i : in std_logic;

      -- Sync input for the trigger unit
      sync_i   : in std_logic;

      -- Generated trigger pulse (after delay, before pad).
      trig_o   : out std_logic;

      --  dff reset (pad)
      trig_rst_n_o : out std_logic;
      trig_rst_p_o : out std_logic;

      --  Asynchronous inputs.
      start_i  : in  std_logic;
      stop_i   : in  std_logic
    );
end vtu_blk;

architecture rtl of vtu_blk is
    -- Delayed sync_i signal.
    signal sync_in_del : std_logic;

    --  Delay value for sync and write signal.
    signal sync_idelay_in  : std_logic_vector (4 downto 0);
    signal sync_idelay_wr  : std_logic;

    --  Sync delay current value.
    signal sync_idelay_out : std_logic_vector (4 downto 0);

    type t_vtu_params is record
        mode : std_logic_vector (2 downto 0);
        htswitchen : std_logic;
        bvalue : std_logic_vector (63 downto 0);
        htvalue : std_logic_vector (63 downto 0);
        wvalue : std_logic_vector (63 downto 0);
    end record;

    constant c_no_vtu_params : t_vtu_params :=
       (mode => (others => '0'),
        htswitchen => '0',
        bvalue => (others => '0'),
        htvalue => (others => '0'),
        wvalue => (others => '0'));

    --  VTU offline parameters.
    signal params_offline : t_vtu_params;
    signal valid_offline : std_logic;
    signal valid_data    : std_logic;
    signal valid_wr      : std_logic;

    --  VTU inline parameters (for clk_sys and clk_vtu domains)
    signal params_vtu : t_vtu_params;
    signal params_online : t_vtu_params;

    signal vtu_use_sync_as_start : std_logic := '0';

    --  Status from VTUcore and their synchronized version.
    signal vtu_wrongb           : std_logic;
    signal vtu_wrongb_sys       : std_logic;
    signal vtu_wronght          : std_logic;
    signal vtu_wronght_sys      : std_logic;
    signal vtu_wrongw           : std_logic;
    signal vtu_wrongw_sys       : std_logic;

    signal start_sys       : std_logic;
    signal start_pulse_sys : std_logic;
    signal start_pulse_2   : std_logic;
    signal start_ready     : std_logic;
    signal start_ack       : std_logic;

    signal vtu_start       : std_logic;
    signal vtu_start_d     : std_logic;
    signal vtu_start_d2     : std_logic;

    signal miss_ready      : std_logic;
    signal miss_valid      : std_logic;

    signal soft_stop_ready : std_logic;
    signal soft_stop_vtu   : std_logic;
    signal stop_in_vtu     : std_logic;
    signal vtu_stop        : std_logic;

    signal soft_start      : std_logic;
    signal soft_stop       : std_logic;
    signal rst_vtu         : std_logic;

    --  Status bit (rf and sys clock domains)
    signal vtu_run         : std_logic;
    signal vtu_run_sys     : std_logic;

    signal sync_data_in    : std_logic_vector(7 downto 0);
    signal sync_data_out   : std_logic_vector(7 downto 0);

    --  Trigger out, before delay.
    signal trig_out       : std_logic;

    --  Delay value for trig and write signal.
    signal trig_odelay_in  : std_logic_vector (4 downto 0);
    signal trig_odelay_wr  : std_logic;

    --  Trig delay current value.
    signal trig_odelay_out : std_logic_vector (4 downto 0);

    signal diag_enable : std_logic;
    signal diag_window : std_logic_vector (2 downto 0);
    signal diag_generation : std_logic_vector (15 downto 0);
    signal diag_freq : std_logic_vector (31 downto 0);
    signal diag_counter : std_logic_vector (31 downto 0);

    signal rst_sys_n : std_logic;
    signal rst_vtu_n : std_logic;

    signal clk_rf_n : std_logic;

    attribute keep_hierarchy                : STRING;
    attribute keep_hierarchy of inst_vtu    : label is "yes";

begin
    rst_sys_n <= not rst_sys_i;
    rst_vtu_n <= not rst_vtu;

    clk_rf_n <= not clk_rf_i;
    
    --  Synchronize start_i
    --  Path: start_i (async)
    --     --> start_pulse_sys (clk_sys)
    --     --> start_pulse_2  (clk_sys, if ready)
    --     --> vtu_start (clk_vtu)
    --     --> vtu_start_d (clk_vtu, after loading the config)
    --         [read by the vtu]
    inst_start_isync: entity work.gc_sync
        port map (
            clk_i     => clk_sys_i,
            rst_n_a_i => rst_sys_n,
            d_i    => start_i,
            q_o   => start_sys
        );
    inst_start_edge: entity work.gc_edge_detect
      generic map (
        g_async_rst => False,
        g_pulse_edge => "positive",
        g_clock_edge => "positive"
      )
      port map (
        clk_i => clk_sys_i,
        rst_n_i => rst_sys_n,
        data_i => start_sys,
        pulse_o => start_pulse_sys
      );
    --  Synchronizer from the external/soft start to the start for VTU.
    inst_start_psync: entity work.gc_pulse_synchronizer2
        port map (
            clk_in_i    => clk_sys_i,
            rst_in_n_i  => rst_sys_n,
            clk_out_i   => clk_vtu_i,
            rst_out_n_i => rst_vtu_n,
            d_ready_o   => start_ready,
            d_ack_p_o   => start_ack,
            d_p_i       => start_pulse_2,
            q_p_o       => vtu_start
        );

    --  FIXME: add a miss flag for soft_stop ?
    inst_softstop_psync: entity work.gc_pulse_synchronizer2
        port map (
            clk_in_i    => clk_sys_i,
            rst_in_n_i  => rst_sys_n,
            clk_out_i   => clk_vtu_i,
            rst_out_n_i => rst_vtu_n,
            d_ready_o   => soft_stop_ready,
            d_ack_p_o   => open,
            d_p_i       => soft_stop,
            q_p_o       => soft_stop_vtu
        );

    inst_stop_psync: entity work.gc_sync
        port map (
            clk_i     => clk_vtu_i,
            rst_n_a_i => rst_vtu_n,
            d_i       => stop_i,
            q_o       => stop_in_vtu
        );

    vtu_stop <= stop_in_vtu or soft_stop_vtu;

    --  Handle start pulse.
    process (clk_sys_i)
    begin
        if rising_edge (clk_sys_i) then
            if rst_sys_n = '0' then
                start_pulse_2 <= '0';
                miss_ready <= '0';
                miss_valid <= '0';
                params_online <= c_no_vtu_params;
            else
                start_pulse_2 <= '0';
                if start_pulse_sys = '1' or soft_start = '1' then
                    --  Start.
                    if valid_offline = '1' and start_ready = '1' then
                        --  If offline config is ready, copy it to online,
                        --  gen a pulse for vtu, gen an interrupt to vme.
                        start_pulse_2 <= '1';
                        --  Clear miss status.  Sticky version ?
                        miss_ready <= '0';
                        miss_valid <= '0';
                        params_online <= params_offline;
                    else
                        --  If not ready, set flags.
                        miss_ready <= not start_ready;
                        miss_valid <= not valid_offline;
                    end if;
                end if;
                if start_ack = '1' then
                    --  Copy for online registers.
                    --  Clear valid
                    valid_offline <= '0';
                elsif valid_wr = '1' then
                    --  Write to VALID register from WB.
                    --  TODO: do not allow change of offline data if the valid bit is set.
                    --  TODO: detect ingored writes to offline data if the valid bit is set.
                    valid_offline <= valid_data;
                end if;
            end if;
        end if;
    end process;

    process (clk_vtu_i)
    begin
        if rising_edge (clk_vtu_i) then
            if rst_vtu_n = '0' then
                params_vtu <= c_no_vtu_params;
                vtu_start_d <= '0';
                vtu_start_d2 <= '0';
            else
                vtu_start_d <= '0';
                --  Delayed 2 is used by the vtu so that it has enough time to check the values.
                vtu_start_d2 <= vtu_start_d;
                if vtu_start = '1' then
                    if valid_offline = '1' then
                        --  CDC
                        --  Registers can be read because they aren't modified
                        --  while the start pulse is transfered.
                        params_vtu <= params_offline;
                        --  Generate a pulse to the vtu.
                        vtu_start_d <= '1';
                    else
                        --  Parameters are not valid -> set to 0.
                        params_vtu <= c_no_vtu_params;
                    end if;
                end if;
            end if;
        end if;
    end process;

    --  VTU status synchronizers
    inst_run_sync: entity work.gc_sync
        port map (
            clk_i => clk_sys_i,
            rst_n_a_i => rst_sys_n,
            d_i => vtu_run,
            q_o => vtu_run_sys);

    inst_wrongb_sync: entity work.gc_sync
        port map (
            clk_i => clk_sys_i,
            rst_n_a_i => rst_sys_n,
            d_i => vtu_wrongb,
            q_o => vtu_wrongb_sys);

    inst_wronght_sync: entity work.gc_sync
        port map (
            clk_i => clk_sys_i,
            rst_n_a_i => rst_sys_n,
            d_i => vtu_wronght,
            q_o => vtu_wronght_sys);

    inst_wrongw_sync: entity work.gc_sync
        port map (
            clk_i => clk_sys_i,
            rst_n_a_i => rst_sys_n,
            d_i => vtu_wrongw,
            q_o => vtu_wrongw_sys);

    inst_trig_diag: entity work.vtu_diag
      port map (
        clk_i => clk_sys_i,
        rst_n_i => rst_sys_n,
        enable_i => diag_enable,
        windows_i => diag_window,
        sig_i => clk_vtu_i,
        dat_i  => sync_data_out,
        freq_o => diag_freq,
        generation_o => diag_generation,
        counter_o => diag_counter
      );

    inst_trigunit_regs: entity work.trigunit_regs
        port map (
            rst_n_i => rst_sys_n,
            clk_i => clk_sys_i,
            wb_i => wb_i,
            wb_o => wb_o,

            status_running_i => vtu_run_sys,
            status_wrongbvalue_i => vtu_wrongb_sys,
            status_wrongwvalue_i => vtu_wrongw_sys,
            status_wronghtvalue_i => vtu_wronght_sys,
            status_startReady_i => start_ready,
            status_missValid_i => miss_valid,
            status_missReady_i => miss_ready,

            control_vtuReset_o         => rst_vtu,
            control_startSoftTrigger_o => soft_start,
            control_stopSoftTrigger_o => soft_stop,

            configoffline_mode_o => params_offline.mode,
            configOffline_htSwitchingEnable_o => params_offline.htswitchen,
            configOffline_valid_i => valid_offline,
            configOffline_valid_o => valid_data,
            configOffline_wr_o => valid_wr,

            bvalueOffline_o => params_offline.bvalue,
            htValueOffline_o => params_offline.htvalue,
            wValueOffline_o => params_offline.wvalue,

            configOnline_mode_i => params_online.mode,
            configOnline_htSwitchingEnable_i => params_online.htswitchen,
            bvalueOnline_i => params_online.bvalue,
            htValueOnline_i => params_online.htvalue,
            wValueOnline_i => params_online.wvalue,

            syncIDelay_delay_i => sync_idelay_out,
            syncIDelay_delay_o => sync_idelay_in,
            syncIdelay_wr_o    => sync_idelay_wr,

            trigODelay_delay_i => trig_odelay_out,
            trigODelay_delay_o => trig_odelay_in,
            trigODelay_wr_o    => trig_odelay_wr,

            trigdiag_control_window_o => diag_window,
            trigdiag_control_enable_o => diag_enable,
            trigdiag_generation_i     => diag_generation,
            trigdiag_freq_i           => diag_freq,
            trigdiag_counter_i        => diag_counter);

    --  The sync pulse comes from a pad
    inst_sync_IDELAYE2 : IDELAYE2
      generic map (
        CINVCTRL_SEL => "FALSE",         -- Enable dynamic clock inversion (FALSE, TRUE)
        DELAY_SRC => "IDATAIN",           -- Delay input (IDATAIN, DATAIN)
        HIGH_PERFORMANCE_MODE => "TRUE", -- Reduced jitter ("TRUE"), Reduced power ("FALSE")
        IDELAY_TYPE => "VAR_LOAD",       -- FIXED, VARIABLE, VAR_LOAD, VAR_LOAD_PIPE
        IDELAY_VALUE => 0,               -- Input delay tap setting (0-31)
        PIPE_SEL => "FALSE",             -- Select pipelined mode, FALSE, TRUE
        REFCLK_FREQUENCY => 200.0,       -- IDELAYCTRL clock input frequency in MHz (190.0-210.0, 290.0-310.0).
        SIGNAL_PATTERN => "DATA"         -- DATA, CLOCK input signal
        )
      port map (
        CNTVALUEOUT => sync_idelay_out,  -- output: Counter value output
        DATAOUT => sync_in_del,          -- output: Delayed data output
        C => clk_sys_i,                  -- input: Clock input
        CE => '0',                       -- input: Active high enable increment/decrement input
        CINVCTRL => '0',                 -- input: Dynamic clock inversion input
        CNTVALUEIN => sync_idelay_in,    -- input: Counter value input
        DATAIN => '0',                   -- input: Internal delay data input
        IDATAIN => sync_i,               -- input: Data input from the I/O
        INC => '0',                      -- input: Increment / Decrement tap delay input
        LD => sync_idelay_wr,            -- input: Load IDELAY_VALUE input
        LDPIPEEN => '0',                 -- input: Enable PIPELINE register to load data input
        REGRST => rst_sys_i);            -- input: Active-high reset tap-delay input

    inst_ISERDESE2 : ISERDESE2
        generic map (
            DATA_RATE => "SDR",              -- DDR, SDR
            DATA_WIDTH => 8,                 -- Parallel data width (2-8,10,14)
            DYN_CLKDIV_INV_EN => "FALSE",    -- Enable DYNCLKDIVINVSEL inversion (FALSE, TRUE)
            DYN_CLK_INV_EN => "FALSE",       -- Enable DYNCLKINVSEL inversion (FALSE, TRUE)
            INIT_Q1 => '0',                  -- INIT_Q1 - INIT_Q4: Initial value on the Q outputs (0/1)
            INIT_Q2 => '0',
            INIT_Q3 => '0',
            INIT_Q4 => '0',
            INTERFACE_TYPE => "NETWORKING",  -- MEMORY, MEMORY_DDR3, MEMORY_QDR, NETWORKING, OVERSAMPLE
            IOBDELAY => "IFD",               -- NONE, BOTH, IBUF, IFD
            NUM_CE => 1,                     -- Number of clock enables (1,2)
            OFB_USED => "FALSE",             -- Select OFB path (FALSE, TRUE)
            SERDES_MODE => "MASTER",         -- MASTER, SLAVE
            SRVAL_Q1 => '0',                 -- SRVAL_Q1 - SRVAL_Q4: Q output values when SR is used (0/1)
            SRVAL_Q2 => '0',
            SRVAL_Q3 => '0',
            SRVAL_Q4 => '0'
            )
        port map (
            O => open,                       -- 1-bit output: Combinatorial output
            Q1 => sync_data_in(0),           -- Q1 - Q8: 1-bit (each) output: Registered data outputs
            Q2 => sync_data_in(1),           -- FIXME: check order
            Q3 => sync_data_in(2),
            Q4 => sync_data_in(3),
            Q5 => sync_data_in(4),
            Q6 => sync_data_in(5),
            Q7 => sync_data_in(6),
            Q8 => sync_data_in(7),
            SHIFTOUT1 => open,               -- SHIFTOUT1, SHIFTOUT2: 1-bit (each) output: Data width expansion output ports
            SHIFTOUT2 => open,
            BITSLIP => '0',                  -- The BITSLIP pin performs a Bitslip operation synchronous
            CE1 => '1',                      -- CE1, CE2: 1-bit (each) input: Data register clock enable inputs
            CE2 => '1',
            CLKDIVP => '0',                  -- 1-bit input: TBD
            CLK => clk_rf_i,                 -- Clocks: 1-bit (each) input: ISERDESE2 clock input ports
            CLKB => clk_rf_n,
            CLKDIV => clk_vtu_i,             -- 1-bit input: Divided clock
            OCLK => '0',                     -- 1-bit input: High speed output clock used when INTERFACE_TYPE="MEMORY"
            DYNCLKDIVSEL => '0',             -- 1-bit input: Dynamic CLKDIV inversion
            DYNCLKSEL => '0',                -- 1-bit input: Dynamic CLK/CLKB inversion
            D => '0',                        -- 1-bit input: Data input
            DDLY => sync_in_del,             -- 1-bit input: Serial data from IDELAYE2
            OFB => '0',                      -- 1-bit input: Data feedback from OSERDESE2
            OCLKB => '0',                    -- 1-bit input: High speed negative edge output clock
            RST => rst_vtu,                  -- 1-bit input: Active high asynchronous reset
            SHIFTIN1 => '0',                 -- SHIFTIN1, SHIFTIN2: 1-bit (each) input: Data width expansion input ports
            SHIFTIN2 => '0'
            );

    inst_vtu : entity work.vtuCore
        port map (
            --  Interface to the memory for the play memory mode.
            Mem_RdDone => '1',
            Mem_Addr => open,
            Mem_RdData => x"00",
            Mem_RdEna => open,

            Mode => params_vtu.mode,

            bValue => params_vtu.bvalue,
            wrongB => vtu_wrongb,
            UseSyncAsStart => vtu_use_sync_as_start,

            htValue => params_vtu.htvalue,
            htSwitching => params_vtu.htswitchen,
            wrongHT => vtu_wronght,

            wValue => params_vtu.wvalue,
            wrongW => vtu_wrongw,

            Start => vtu_start_d2,
            Stop => vtu_stop,

            DataIn => sync_data_in,
            DataOut => sync_data_out,

            Mem_LastBit => (others => '0'),
            Mem_LastElem => (others => '0'),

            SyncPulse => open,
            Rst => rst_vtu,
            Run => vtu_run,
            Clk => clk_vtu_i);

    inst_trig_OSERDESE2 : OSERDESE2
        generic map (
            DATA_RATE_OQ => "SDR",          -- DDR, SDR
            DATA_RATE_TQ => "SDR",          -- DDR, BUF, SDR
            DATA_WIDTH => 8,                -- Parallel data width (2-8,10,14)
            INIT_OQ => '0',                 -- Initial value of OQ output (1'b0,1'b1)
            INIT_TQ => '0',                 -- Initial value of TQ output (1'b0,1'b1)
            SERDES_MODE => "MASTER",        -- MASTER, SLAVE
            SRVAL_OQ => '0',                -- OQ output value when SR is used (1'b0,1'b1)
            SRVAL_TQ => '0',                -- TQ output value when SR is used (1'b0,1'b1)
            TBYTE_CTL => "FALSE",           -- Enable tristate byte operation (FALSE, TRUE)
            TBYTE_SRC => "FALSE",           -- Tristate byte source (FALSE, TRUE)
            TRISTATE_WIDTH => 1             -- 3-state converter width (1,4)
            )
        port map (
            OFB => trig_out,               -- 1-bit output: Feedback path for data
            OQ => open,                     -- 1-bit output: Data path output
            SHIFTOUT1 => open,              -- SHIFTOUT1 / SHIFTOUT2: 1-bit (each) output: Data output expansion (1-bit each)
            SHIFTOUT2 => open,
            TBYTEOUT => open,               -- 1-bit output: Byte group tristate
            TFB => open,                    -- 1-bit output: 3-state control
            TQ => open,                     -- 1-bit output: 3-state control
            CLK => clk_rf_i,               -- 1-bit input: High speed clock
            CLKDIV => clk_vtu_i,            -- 1-bit input: Divided clock
            D1 => sync_data_out(7),         -- D1 - D8: 1-bit (each) input: Parallel data inputs (1-bit each)
            D2 => sync_data_out(6),
            D3 => sync_data_out(5),
            D4 => sync_data_out(4),
            D5 => sync_data_out(3),
            D6 => sync_data_out(2),
            D7 => sync_data_out(1),
            D8 => sync_data_out(0),
            OCE => '1',                     -- 1-bit input: Output data clock enable
            RST => rst_vtu,                 -- 1-bit input: Reset
            SHIFTIN1 => '0',                -- SHIFTIN1 / SHIFTIN2: 1-bit (each) input: Data input expansion (1-bit each)
            SHIFTIN2 => '0',
            T1 => '0',                      -- T1 - T4: 1-bit (each) input: Parallel 3-state inputs
            T2 => '0',
            T3 => '0',
            T4 => '0',
            TBYTEIN => '0',                 -- 1-bit input: Byte group tristate
            TCE => '0'                      -- 1-bit input: 3-state clock enable
        );
    inst_trig_ODELAYE2 : ODELAYE2
        generic map (
            CINVCTRL_SEL => "FALSE",         -- Enable dynamic clock inversion (FALSE, TRUE)
            DELAY_SRC => "ODATAIN",          -- Delay input (ODATAIN, CLKIN)
            HIGH_PERFORMANCE_MODE => "TRUE", -- Reduced jitter ("TRUE"), Reduced power ("FALSE")
            ODELAY_TYPE => "VAR_LOAD",       -- FIXED, VARIABLE, VAR_LOAD, VAR_LOAD_PIPE
            ODELAY_VALUE => 0,               -- Output delay tap setting (0-31)
            PIPE_SEL => "FALSE",             -- Select pipelined mode, FALSE, TRUE
            REFCLK_FREQUENCY => 200.0,       -- IDELAYCTRL clock input frequency in MHz (190.0-210.0, 290.0-310.0).
            SIGNAL_PATTERN => "DATA"         -- DATA, CLOCK input signal
            )
        port map (
            CNTVALUEOUT => trig_odelay_out,  -- output: Counter value output
            DATAOUT => trig_o,               -- output: Delayed data/clock output
            C => clk_sys_i,                  -- input: Clock input
            CE => '0',                       -- input: Active high enable increment/decrement input
            CINVCTRL => '0',                 -- input: Dynamic clock inversion input
            CLKIN => '0',                    -- input: Clock delay input
            CNTVALUEIN => trig_odelay_in,    -- input: Counter value input
            INC => '0',                      -- input: Increment / Decrement tap delay input
            LD => trig_odelay_wr,            -- input: Loads ODELAY_VALUE tap delay in VARIABLE mode
            LDPIPEEN => '0',                 -- input: Enables the pipeline register to load data
            ODATAIN => trig_out ,            -- input: Output delay data input
            REGRST => rst_sys_i              -- input: Active-high reset tap-delay input
        );

    inst_trig_rst_OBUFDS: OBUFDS
        generic map (
            IOSTANDARD => "LVDS" )
        port map (
            I  => '0',     --  FIXME
            O  => trig_rst_p_o,
            OB => trig_rst_n_o );
end rtl;
