--------------------------------------------------------------------------------
-- CERN BE-CO-HT
--
-- https://www.ohwr.org/projects/wr2rf-vme
--------------------------------------------------------------------------------
--
-- unit name:   wr2rf_rftrigger
--
-- description: Provides clock architecture wrapper for VTU
--
--------------------------------------------------------------------------------
-- Copyright CERN 2020
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

use work.wishbone_pkg.all;

entity wr2rf_rftrigger is

    port (
        --  System clock (125Mhz or 62.5Mhz)
        clk_sys_i    : in  std_logic;
        --  Systen reset clock (> 50ns)
        rst_sys_n_i  : in  std_logic;
        rst_sys_i    : in  std_logic;        

        --  Reference clock for io delays
        clk_refclk_200m_i : in std_logic;
        iodelay_reset_i   : in std_logic;

        --  Wishbone interface, uses clk_sys.
        wb_i       : in    t_wishbone_slave_in;
        wb_o       : out   t_wishbone_slave_out;

        --  Pulse from the Reset/sync logic.
        rf_sync_i         : in  std_logic;
        rf_sync_p_o       : inout std_logic;
        rf_sync_n_o       : inout std_logic;

        rf_t1_clk_p_i     : in  std_logic;
        rf_t1_clk_n_i     : in  std_logic;
        rf_t1_p_o         : inout std_logic;
        rf_t1_n_o         : inout std_logic;
        rf_t1_rst_p_o     : out std_logic;
        rf_t1_rst_n_o     : out std_logic;
        rf_t1_start_i     : in  std_logic; --  Asynchronous tmg inputs.
        rf_t1_stop_i      : in  std_logic; --  Asynchronous tmg inputs.

        rf_t2_clk_p_i     : in  std_logic;
        rf_t2_clk_n_i     : in  std_logic;
        rf_t2_p_o         : inout std_logic;
        rf_t2_n_o         : inout std_logic;
        rf_t2_rst_p_o     : out std_logic;
        rf_t2_rst_n_o     : out std_logic;
        rf_t2_start_i     : in  std_logic; --  Asynchronous tmg inputs.
        rf_t2_stop_i      : in  std_logic; --  Asynchronous tmg inputs.

        rf_clk_bufg_o     : out std_logic );

    attribute keep_hierarchy                    : STRING;
    attribute keep_hierarchy of wr2rf_rftrigger : entity is "yes";
    
end entity;

architecture rtl of wr2rf_rftrigger is
  signal rf_clk_in   : std_logic;

  signal rf_clk_io : std_logic;
  signal clk_vtu : std_logic;

  signal rf_idelay_rdy : std_logic;

  signal rf_sync : std_logic;

  signal rf_t1_out : std_logic;
  signal rf_t1_fb : std_logic;

  signal rf_t2_out : std_logic;
  
  signal rst_sys_n_r : std_logic;
  signal rst_sys_r   : std_logic;  

  signal t1_wb_in : t_wishbone_slave_in;
  signal t1_wb_out : t_wishbone_slave_out;

  signal t2_wb_in : t_wishbone_slave_in;
  signal t2_wb_out : t_wishbone_slave_out;

  signal diag_rf_window : std_logic_vector (2 downto 0);
  signal diag_rf_enable : std_logic;
  signal diag_rf_freq : std_logic_vector (31 downto 0);
  signal diag_rf_generation : std_logic_vector (15 downto 0);
  signal diag_rf_counter : std_logic_vector (31 downto 0);

  -- attribute keep_hierarchy                : STRING;
  -- attribute keep_hierarchy of rf_t1_i    : label is "yes";
  -- attribute keep_hierarchy of rf_t2_i    : label is "yes";
begin
  --  Clock.
  inst_rf_clk_IBUFDS: IBUFDS
    generic map (
      DIFF_TERM    => FALSE, --Differential Termination
      IBUF_LOW_PWR => TRUE,  --Lowpower (TRUE) vs. performance (FALSE) setting for referenced I/O standards
      IOSTANDARD   => "DEFAULT")
    port map(
      O => rf_clk_in,
      I => rf_t1_clk_p_i,
      IB => rf_t1_clk_n_i);

  --  Distribute to the I/Os for serdes.
  inst_rf_clk_io_BUFIO: BUFIO
    port map (
      O => rf_clk_io,
      I   => rf_clk_in);

  --  Distribute to the region for logic (vtu).  Note: divided by 8.
  inst_rf_clk_BUFR: BUFR
    generic map (
      BUFR_DIVIDE => "8",
      SIM_DEVICE => "7SERIES")
    port map (
      O => clk_vtu,
      CE  => '1',
      CLR => '0',
      I   => rf_clk_in);

  --  Output the sync signal as a testpoint, and also use the pad as an input
  --  to the trigger unit (through an idelay).
  rf_sync_OBUFDS : IOBUFDS
    generic map (
      IOSTANDARD => "LVDS",
      SLEW => "SLOW")
    port map (
      IO  => rf_sync_p_o,  -- Diff_p output
      IOB => rf_sync_n_o,  -- Diff_n output
      T => '0',
      O => rf_sync,
      I => rf_sync_i);

  process (clk_sys_i)
  begin
     if rising_edge(clk_sys_i) then
        rst_sys_r   <= rst_sys_i;
        rst_sys_n_r <= rst_sys_n_i;
     end if;
  end process;

  rf_t1_i : entity work.vtu_blk
    port map (
      clk_sys_i        => clk_sys_i,
      rst_sys_i        => rst_sys_r,
      wb_i             => t1_wb_in,
      wb_o             => t1_wb_out,

      clk_rf_i         => rf_clk_io,
      clk_vtu_i        => clk_vtu,
      sync_i           => rf_sync,
      trig_o           => rf_t1_out,
      trig_rst_p_o     => rf_t1_rst_p_o,
      trig_rst_n_o     => rf_t1_rst_n_o,
      start_i          => rf_t1_start_i,
      stop_i           => rf_t1_stop_i);

  --  Unit 1 output
  rf_trig_OBUFDS : OBUFDS
    generic map (
      IOSTANDARD => "LVDS",
      SLEW => "SLOW")
    port map (
      O  => rf_t1_p_o,  -- Diff_p output
      OB => rf_t1_n_o,  -- Diff_n output
      I => rf_t1_out);

  --  Feedback (directly from the pad) of unit 1 output.
  rf_trig_IBUFDS : IBUFDS
    generic map (
      DIFF_TERM    => FALSE, --Differential Termination
      IBUF_LOW_PWR => TRUE,  --Lowpower (TRUE) vs. performance (FALSE) setting for referenced I/O standards
      IOSTANDARD   => "LVDS")
    port map (
      I  => rf_t1_p_o,  -- Diff_p output
      IB => rf_t1_n_o,  -- Diff_n output
      O   => rf_t1_fb);

  rf_t2_i : entity work.vtu_blk
      port map (
        clk_sys_i        => clk_sys_i,
        rst_sys_i        => rst_sys_r,
        wb_i             => t2_wb_in,
        wb_o             => t2_wb_out,
  
        clk_rf_i         => rf_clk_io,
        clk_vtu_i        => clk_vtu,
        sync_i           => rf_t1_fb,
        trig_o           => rf_t2_out,
        trig_rst_p_o     => rf_t2_rst_p_o,
        trig_rst_n_o     => rf_t2_rst_n_o,
        start_i          => rf_t2_start_i,
        stop_i           => rf_t2_stop_i);
  
  --  Unit 2 output
  rf_trig2_OBUFDS : OBUFDS
    generic map (
      IOSTANDARD => "LVDS",
      SLEW => "SLOW")
    port map (
      O  => rf_t2_p_o,  -- Diff_p output
      OB => rf_t2_n_o,  -- Diff_n output
      I => rf_t2_out);

  --  Distribute clock to a bufg (for P0 signals)
  rf_bufg_1 : BUFG
    port map (
      I => rf_clk_in,
      O => rf_clk_bufg_o);

  --  Delay control (1 per clock region)
  rf_IDELAYCTRL : IDELAYCTRL
    port map (
      RDY    => rf_idelay_rdy,     -- 1-bit output: Ready output
      REFCLK => clk_refclk_200m_i, -- 1-bit input: Reference clock input
      RST    => iodelay_reset_i ); -- 1-bit input: Active high reset input

  --  Diag for rf_clk.
  inst_rf_diag: entity work.vtu_diag
  port map (
    clk_i => clk_sys_i,
    rst_n_i => rst_sys_n_r,
    sig_i => rf_clk_in,
    dat_i => b"0100_0000",  -- One pulse.
    enable_i => diag_rf_enable,
    windows_i => diag_rf_window,
    freq_o => diag_rf_freq,
    generation_o => diag_rf_generation,
    counter_o => diag_rf_counter
  );


  inst_regs: entity work.wr2rf_rftrigger_regs
    port map (
      rst_n_i => rst_sys_n_r,
      clk_i => clk_sys_i,
      wb_i => wb_i,
      wb_o => wb_o,
      t1_i => t1_wb_out,
      t1_o => t1_wb_in,
      t2_i => t2_wb_out,
      t2_o => t2_wb_in,
      rf_diag_control_enable_o => diag_rf_enable,
      rf_diag_control_window_o => diag_rf_window,
      rf_diag_freq_i => diag_rf_freq,
      rf_diag_generation_i => diag_rf_generation,
      rf_diag_counter_i => diag_rf_counter
    );
end architecture;
