--------------------------------------------------------------------------------
-- CERN BE-CO-HT
-- WR2RF_VME core
-- https://ohwr.org/project/vme-rf-wr-bobr
--------------------------------------------------------------------------------
--
-- unit name:   wr2rf_vme
--
-- description: top level for the wr2rf_vme board
--
--------------------------------------------------------------------------------
-- Copyright (c) 2019 CERN (home.cern)
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.vme64x_pkg.all;
use work.wrcore_pkg.all;
use work.wr_fabric_pkg.all;
use work.wr_board_pkg.all;
use work.streamers_pkg.all;
use work.RFFrameTransceiver_pkg.all;

entity wr2rf_vme is

  generic (
    g_simulation          : integer := 0;
    g_dpram_size          : integer := 131072/4;
--    g_dpram_initf         : string  := "../../../../dependencies/wrpc-sw/wrc.bram";
    g_dpram_initf         : string  := "";    
    g_diag_id             : integer := 0;
    g_diag_ver            : integer := 0;
    g_diag_ro_size        : integer := 0;
    g_diag_rw_size        : integer := 0;
    g_fabric_iface        : t_board_fabric_iface := STREAMERS;
    g_streamers_op_mode   : t_streamers_op_mode  := TX_AND_RX;
    g_tx_streamer_params  : t_tx_streamer_params := c_tx_streamer_params_RF;
    g_rx_streamer_params  : t_rx_streamer_params := c_rx_streamer_params_RF;
    g_ftw_length          : natural := 48;
    g_offset_length       : natural := 48  );

  port (
    ---------------------------------------------------------------------------
    -- VME interface D16 A24
    -- Note: retry, berr not required
    --       addr is input only.
    ---------------------------------------------------------------------------

    vme_write_n_i    : in    std_logic;
    vme_sysreset_n_i : in    std_logic;
    vme_retry_oe_o   : out   std_logic;
    vme_retry_n_o    : out   std_logic;
    vme_lword_n_b    : inout std_logic;
    vme_iackout_n_o  : out   std_logic;
    vme_iackin_n_i   : in    std_logic;
    vme_iack_n_i     : in    std_logic;
    vme_dtack_oe_o   : out   std_logic;
    vme_dtack_n_o    : out   std_logic;
    vme_ds_n_i       : in    std_logic_vector(1 downto 0);
    vme_data_oe_n_o  : out   std_logic;
    vme_data_dir_o   : out   std_logic;
    vme_berr_o       : out   std_logic;
    vme_as_n_i       : in    std_logic;
    vme_irq_o        : out   std_logic_vector(7 downto 1);
    vme_data_b       : inout std_logic_vector(15 downto 0);
    vme_am_i         : in    std_logic_vector(5 downto 0);
    vme_addr_b       : inout std_logic_vector(23 downto 1);
    vme_addr_dir_o   : out   std_logic;
    vme_addr_oe_n_o  : out   std_logic;

    --  Geographic address
    vme_gap_i        : in    std_logic;
    vme_ga_n_i       : in    std_logic_vector(4 downto 0);

    --  Switches in absence of vme64x.
    vme_noga_n_i     : in    std_logic_vector(4 downto 0);
    switch_n         : in    std_logic_vector(2 downto 0);

    -- 62.5MHz + offset clock for DDMTD.
    clk_dmtd_62m5_p_i : in std_logic;
    clk_dmtd_62m5_n_i : in std_logic;

    -- 125 MHz Clock for Xilinx GTX transceiver.
    clk_125m_gtx_n_i : in    std_logic;
    clk_125m_gtx_p_i : in    std_logic;

    -- 62.5 MHz system clock from
    clk_sys_62m5_p_i : in std_logic;
    clk_sys_62m5_n_i : in std_logic;

    -- 10 MHz external reference clock.
    clk_ext_10m_p_i : in std_logic;
    clk_ext_10m_n_i : in std_logic;
    clk_ext_10m_o   : out std_logic;
    ext_ref_dir_o   : out std_logic;

    -- 62.5 MHz external reference clock (the above * 6.25)
--    clk_ext_62m5_p_i : in std_logic;
--    clk_ext_62m5_n_i : in std_logic;

    --  SFP 1
    sfp1_tx_n_o        : out   std_logic;
    sfp1_tx_p_o        : out   std_logic;
    sfp1_rx_n_i        : in    std_logic;
    sfp1_rx_p_i        : in    std_logic;
    sfp1_tx_fault_i    : in    std_logic;
    sfp1_los_i         : in    std_logic;
    sfp1_tx_disable_o  : out   std_logic;

    sfp1_led_active_o  : out   std_logic;
    sfp1_led_link_o    : out   std_logic;
    sfp1_detect_i      : in    std_logic;
    sfp1_rate_select_o : out   std_logic;
    sfp1_scl_b         : inout std_logic;
    sfp1_sda_b         : inout std_logic;

    --  SFP 2
    --sfp2_tx_n_o        : out   std_logic;
    --sfp2_tx_p_o        : out   std_logic;
    --sfp2_rx_n_i        : in    std_logic;
    --sfp2_rx_p_i        : in    std_logic;
    sfp2_tx_fault_i    : in    std_logic;
    sfp2_los_i         : in    std_logic;
    sfp2_tx_disable_o  : out   std_logic;

    sfp2_led_active_o  : out   std_logic;
    sfp2_led_link_o    : out   std_logic;
    sfp2_detect_i      : in    std_logic;
    sfp2_rate_select_o : out   std_logic;
    sfp2_scl_b         : inout std_logic;
    sfp2_sda_b         : inout std_logic;

    -- PPS IN & OUT
    pps_i : in  std_logic;
    pps_o : out std_logic;

    -- dac ddmtd offset clock
    wr_dac_dmtd_sclk_o   : out std_logic;
    wr_dac_dmtd_din_o    : out std_logic;
    wr_dac_dmtd_cs_n_o   : out std_logic;

    -- dac internal WR timebase
    wr_dac_ocxo_sclk_o   : out std_logic;
    wr_dac_ocxo_din_o    : out std_logic;
    wr_dac_ocxo_cs_n_o   : out std_logic;

    -- Firmware flash
    spi_flash_cs_n_o   : out std_logic;
    spi_flash_miso_i   : in  std_logic;
    spi_flash_mosi_o   : out std_logic;

    --  WR eeprom
    wr_eeprom1_scl_b       : inout std_logic;
    wr_eeprom1_sda_b       : inout std_logic;
    wr_eeprom2_scl_b       : inout std_logic;
    wr_eeprom2_sda_b       : inout std_logic;

    -- Onewire
    wr_onewire_b       : inout std_logic;

    -- OCXO
    ocxo_en_o          : out   std_logic;

    -- timing_io
    tmg_clk_i          : in    std_logic_vector(2 downto 1);
    tmg_clk_o          : out   std_logic_vector(2 downto 1);
    tmg_clk_oe_n_o     : out   std_logic_vector(2 downto 1);
    tmg_clk_term_en_o  : out   std_logic_vector(2 downto 1);
    tmg_io_i           : in    std_logic_vector(4 downto 1);
    tmg_io_o           : out   std_logic_vector(4 downto 1);
    tmg_io_oe_n_o      : out   std_logic_vector(4 downto 1);
    tmg_io_term_en_o   : out   std_logic_vector(4 downto 1);

    -- rf1 iqdac
    rf1_iqdac_data_p_o : out std_logic_vector(15 downto 0);
    rf1_iqdac_data_n_o : out std_logic_vector(15 downto 0);
    rf1_iqdac_dci_p_o  : out std_logic;
    rf1_iqdac_dci_n_o  : out std_logic;
    rf1_iqdac_reset_o  : out std_logic;

    -- rf2 iqdac
    rf2_iqdac_data_p_o : out std_logic_vector(15 downto 0);
    rf2_iqdac_data_n_o : out std_logic_vector(15 downto 0);
    rf2_iqdac_dci_p_o  : out std_logic;
    rf2_iqdac_dci_n_o  : out std_logic;
    rf2_iqdac_reset_o  : out std_logic;

    -- rf1 channel
    rf1_sync_p_o       : inout std_logic;
    rf1_sync_n_o       : inout std_logic;

    -- rf1 trigger1 channel
    rf1_t1_clk_p_i     : in  std_logic;
    rf1_t1_clk_n_i     : in  std_logic;
    rf1_t1_p_o         : inout std_logic;
    rf1_t1_n_o         : inout std_logic;
    rf1_t1_rst_p_o     : out std_logic;
    rf1_t1_rst_n_o     : out std_logic;

    -- rf1 trigger2 channel
    rf1_t2_clk_p_i     : in  std_logic;
    rf1_t2_clk_n_i     : in  std_logic;
    rf1_t2_p_o         : inout std_logic;
    rf1_t2_n_o         : inout std_logic;
    rf1_t2_rst_p_o     : out std_logic;
    rf1_t2_rst_n_o     : out std_logic;

    -- rf2 channel
    rf2_sync_p_o       : inout std_logic;
    rf2_sync_n_o       : inout std_logic;

    -- rf2 trigger1 channel
    rf2_t1_clk_p_i     : in  std_logic;
    rf2_t1_clk_n_i     : in  std_logic;
    rf2_t1_p_o         : inout std_logic;
    rf2_t1_n_o         : inout std_logic;
    rf2_t1_rst_p_o     : out std_logic;
    rf2_t1_rst_n_o     : out std_logic;

    -- rf2 trigger2 channel
    rf2_t2_clk_p_i     : in  std_logic;
    rf2_t2_clk_n_i     : in  std_logic;
    rf2_t2_p_o         : inout std_logic;
    rf2_t2_n_o         : inout std_logic;
    rf2_t2_rst_p_o     : out std_logic;
    rf2_t2_rst_n_o     : out std_logic;

    -- vme p0
    vme_p0_h1_bsb_o    : out std_logic_vector(7 downto 0);
    vme_p0_h1_c40_o    : out std_logic;
    vme_p0_h1_lbb_o   : out std_logic_vector(7 downto 0);
    vme_p0_h1_hbb_o   : out std_logic_vector(7 downto 0);
    vme_p0_h1_tc_o     : out std_logic;
    vme_p0_h2_bsb_o    : out std_logic_vector(7 downto 0);
    vme_p0_h2_c40_o    : out std_logic;
    vme_p0_h2_lbb_o   : out std_logic_vector(7 downto 0);
    vme_p0_h2_hbb_o   : out std_logic_vector(7 downto 0);
    vme_p0_h2_tc_o     : out std_logic;
    vme_p0_oen_o       : out std_logic;

    --  SPI for DDS and DACs
    dds_cs_n_o         : out std_logic;
    rf1_iqdac_cs_n_o   : out std_logic;
    rf2_iqdac_cs_n_o   : out std_logic;
    rf_sclk_o          : out std_logic;
    rf_sdi_o           : out std_logic;
    rf_sdo_i           : in  std_logic;

    -- DDS control
    dds_profile_o      : out std_logic_vector(2 downto 0);
    dds_ioupdate_o     : out std_logic;
    dds_sync_p_o       : out std_logic;
    dds_sync_n_o       : out std_logic;
    dds_sync_error_i   : in  std_logic;
    dds_reset_o        : out std_logic;

    -- SPI for PLL (ltc6950)
    pll_main_cs_n_o : out std_logic;
    pll_main_sclk_o : out std_logic;
    pll_main_sdi_o  : out std_logic;
    pll_main_sdo_i  : in  std_logic;
    pll_main_stat_i : in  std_logic_vector(2 downto 1);
    pll_main_sync_o : out std_logic;

    --  RF 1
    rf1_mux_sel_o        : out std_logic_vector(1 downto 0);
    rf1_mixer_en_o       : out std_logic;
    rf1_t1_delay_latch_o : out std_logic;
    rf1_t1_delay_oen_o   : out std_logic;
    rf1_t1_mux_sel_o     : out std_logic;
    rf1_t2_delay_latch_o : out std_logic;
    rf1_t2_delay_oen_o   : out std_logic;
    rf1_t2_mux_sel_o     : out std_logic;

    --  RF 2
    rf2_mux_sel_o        : out std_logic_vector(1 downto 0);
    rf2_mixer_en_o       : out std_logic;
    rf2_t1_delay_latch_o : out std_logic;
    rf2_t1_delay_oen_o   : out std_logic;
    rf2_t1_mux_sel_o     : out std_logic;
    rf2_t2_delay_latch_o : out std_logic;
    rf2_t2_delay_oen_o   : out std_logic;
    rf2_t2_mux_sel_o     : out std_logic;

    rf_delay_o           : out std_logic_vector(9 downto 0) );

end wr2rf_vme;

architecture rtl of wr2rf_vme is

  constant c_pcs_data_width : integer := 16;

  subtype t_poweron_counter is natural range 0 to 15;
  signal poweron_counter : t_poweron_counter := t_poweron_counter'high;
  signal rst_sys_n_r     : std_logic;
  signal rst_sys_r       : std_logic;
  signal rst_sys_n       : std_logic;
  signal rst_sys         : std_logic;

  signal clk62m5          : std_logic;
  signal clk125m          : std_logic;
  signal clk200m          : std_logic;
  signal clk250m          : std_logic;
  signal rst_clk62m5_n    : std_logic;
  signal rst_clk62m5      : std_logic;
  signal rst_clk125m_n    : std_logic;
  signal rst_clk125m      : std_logic;
  signal rst_clk200m_n    : std_logic;
  signal rst_clk200m      : std_logic;
  signal rst_clk250m_n    : std_logic;
  signal rst_clk250m      : std_logic;


  -- Wishbone bus from master
  signal wb_vme_out : t_wishbone_master_out;
  signal wb_vme_in  : t_wishbone_master_in;

  -- VME
  signal vme_data_b_out    : std_logic_vector(15 downto 0);
  signal vme_data_b_out1   : std_logic_vector(31 downto 16);
  signal vme_addr_b_out    : std_logic_vector(31 downto 1);
  signal vme_lword_n_b_out : std_logic;
  signal vme_data_dir_int  : std_logic;
  signal vme_addr_dir_int  : std_logic;
  signal vme_ga            : std_logic_vector(5 downto 0);
  signal vme_berr_n_o      : std_logic;
  signal vme_irq_n_o       : std_logic_vector(7 downto 1);

  signal vme_nogap         : std_logic;

  signal clk_dmtd_62m5     : std_logic;
  signal clk_sys_62m5      : std_logic;
  signal clk_sys_62m5_in   : std_logic;
--  signal clk_ext_62m5      : std_logic;
  signal clk_ext_10m_in    : std_logic;
  signal clk_gtx_125m      : std_logic;

  signal clk_sys_select    : std_logic;
  signal clk_sys_reset     : std_logic;

  signal gtx_qpll_ref_clk, gtx_qpll_clk, gtx_qpll_locked : std_logic;
  signal gtx_qpll_reset                                  : std_logic;

  signal phy_tx_data      : std_logic_vector(15 downto 0);
  signal phy_tx_k         : std_logic_vector(1 downto 0);
  signal phy_tx_disparity : std_logic;
  signal phy_tx_enc_err   : std_logic;
  signal phy_rx_data      : std_logic_vector(15 downto 0);
  signal phy_rx_rbclk     : std_logic;
  signal phy_rx_k         : std_logic_vector(1 downto 0);
  signal phy_rx_enc_err   : std_logic;
  signal phy_rst          : std_logic;
  signal phy_loopen       : std_logic;
  --loopen_i determines (7 Series Transceiver User Guide(UG476) Figure 2-23 and Table 2-37):
  --'0' => gtx_loopback = "000" => normal operation
  --'1' => gtx_loopback = "100" => Far-end PMA Loopback
  signal phy_loopen_vec   : std_logic_vector(2 downto 0);
  signal phy_prbs_sel     : std_logic_vector(2 downto 0);
  signal phy_rdy          : std_logic;

  signal phy_rx_rbclk_sampled : std_logic;

  signal phy_lpc_ctrl  : std_logic_vector(15 downto 0);
  signal phy_lpc_stat  : std_logic_vector(15 downto 0);

  signal pps_p     : std_logic;
  signal pps_csync : std_logic;

  signal dac_hpll_load_p1_o : std_logic;
  signal dac_dpll_load_p1_o : std_logic;
  signal dac_hpll_data_o    : std_logic_vector(15 downto 0);
  signal dac_dpll_data_o    : std_logic_vector(15 downto 0);

  signal sfp1_scl_out : std_logic;
  signal sfp1_scl_in  : std_logic;
  signal sfp1_sda_out : std_logic;
  signal sfp1_sda_in  : std_logic;

  signal wr1_scl_out : std_logic;
  signal wr1_scl_in  : std_logic;
  signal wr1_sda_out : std_logic;
  signal wr1_sda_in  : std_logic;

  signal ocxo_enabled : std_logic;
  signal ocxo_uptime  : std_logic_vector(15 downto 0);

  signal owr_en : std_logic_vector(1 downto 0);
  signal owr_i  : std_logic_vector(1 downto 0);

  signal wb_wrc_in  : t_wishbone_master_out;
  signal wb_wrc_out : t_wishbone_master_in;

  signal rf1_clk_bufg     : std_logic;
  signal rf2_clk_bufg     : std_logic;

  signal rf1_sync : std_logic;
  signal rf2_sync : std_logic;

  signal mmcm_locked      : std_logic;
  signal wrcore_reset_n   : std_logic;
  signal iodelay_reset    : std_logic;

  signal wb_rf1_vtus_in       : t_wishbone_master_in;
  signal wb_rf1_vtus_out      : t_wishbone_master_out;

  signal wb_rf2_vtus_in       : t_wishbone_master_in;
  signal wb_rf2_vtus_out      : t_wishbone_master_out;

  signal wb_rf_framerxtx_in   : t_wishbone_master_in;
  signal wb_rf_framerxtx_out  : t_wishbone_master_out;

  signal mmcm_shift_incdec : std_logic;
  signal mmcm_shift_en     : std_logic;
  signal mmcm_shift_busy   : std_logic;

  signal tmg_clk_oe_o  : std_logic_vector(2 downto 1);
  signal tmg_io_oe_o   : std_logic_vector(4 downto 1);

  signal pps                   : std_logic;

  signal tm_link_up    : std_logic;
  signal tm_time_valid : std_logic;
  signal tm_tai        : std_logic_vector(39 downto 0);
  signal tm_cycles     : std_logic_vector(27 downto 0);

  -- WR fabric interface
  signal wrf_src_out : t_wrf_source_out;
  signal wrf_src_in  : t_wrf_source_in;
  signal wrf_snk_out : t_wrf_sink_out;
  signal wrf_snk_in  : t_wrf_sink_in;

  signal wrs_tx_data  : std_logic_vector(g_tx_streamer_params.data_width-1 downto 0) := (others => '0');
  signal wrs_tx_valid : std_logic                                        := '0';
  signal wrs_tx_dreq  : std_logic;
  signal wrs_tx_last  : std_logic                                        := '1';
  signal wrs_tx_flush : std_logic                                        := '0';
  signal wrs_rx_first : std_logic;
  signal wrs_rx_last  : std_logic;
  signal wrs_rx_data  : std_logic_vector(g_rx_streamer_params.data_width-1 downto 0);
  signal wrs_rx_valid : std_logic;
  signal wrs_rx_dreq  : std_logic                                        := '0';
  signal wrs_tx_cfg   : t_tx_streamer_cfg := c_tx_streamer_cfg_default;
  signal wrs_rx_cfg   : t_rx_streamer_cfg := c_rx_streamer_cfg_default;

  signal link_ok      : std_logic;

  signal wb_aux_out   : t_wishbone_master_out;
  signal wb_aux_in    : t_wishbone_master_in;
--  signal aux_rst_n      : std_logic;

  constant c_streamers_diag_id  : integer := 1;  -- id reserved for streamers
  constant c_streamers_diag_ver : integer := 2;  -- version that will be probably increased
  constant c_diag_id            : integer := f_pick_diag_val(STREAMERS,  c_streamers_diag_id,         g_diag_id);
  constant c_diag_ver           : integer := f_pick_diag_val(STREAMERS,  c_streamers_diag_ver,        g_diag_id);
  constant c_diag_ro_size       : integer := f_pick_diag_size(g_fabric_iface, c_WR_STREAMERS_ARR_SIZE_OUT, g_diag_ro_size);
  constant c_diag_rw_size       : integer := f_pick_diag_size(g_fabric_iface, c_WR_STREAMERS_ARR_SIZE_IN, g_diag_rw_size);

  -- WR SNMP
  signal aux_diag_in  : t_generic_word_array(c_diag_ro_size-1 downto 0);
  signal aux_diag_out : t_generic_word_array(c_diag_rw_size-1 downto 0);

  -- RFFrameTransceiver
  signal wrs_rx_RFFrameHeader    : t_RFFrameHeader;
  signal wrs_rx_RFmFramePayloads : t_RFmFramePayload;
  signal wrs_rx_RFsFramePayloads : t_RFsFramePayload;
  signal wrs_rx_Frame_valid_pX   : std_logic;
  signal wrs_rx_Frame_typeID     : std_logic_vector(c_RFtype_ID_size-1 downto 0);
  signal wrs_tx_ready            : std_logic;
  signal wrs_tx_TransmitFrame_p1 : std_logic         := '0';
  signal wrs_tx_RFFrameHeader    : t_RFFrameHeader   := c_RFFrameHeader_zero;
  signal wrs_tx_RFmFramePayloads : t_RFmFramePayload := c_RFmFramePayload_zero;
  signal wrs_tx_RFsFramePayloads : t_RFsFramePayload := c_RFsFramePayload_zero;
  signal loc_rx_RFmFramePayloads : t_RFmFramePayload;
  signal loc_rx_Frame_Valid_pX   : std_logic;
  signal rx_RFmFramePayloads     : t_RFmFramePayload;
  signal rx_Frame_Valid_pX       : std_logic;
  signal rx_Frame_typeID         : std_logic_vector(7 downto 0);
  signal rx_reset_nco_pulse      : std_logic;

  signal loc_nco_ctrl_reset      : std_logic;
  signal loc_nco_h1_ftw          : std_logic_vector(c_FTW_size-1 downto 0);
  signal loc_nco_h1_prog         : std_logic_vector(c_FTW_size-1 downto 0);
  signal loc_nco_update_valid    : std_logic;
  signal loc_or_wrs_params_sel   : std_logic;
  signal loc_nco_update_valid_r  : std_logic;
  signal loc_rx_reset_nco_pulse  : std_logic;

  signal loc_nco_h1_ftw_r        : std_logic_vector(c_FTW_size-1 downto 0);
  signal loc_nco_h1_prog_r       : std_logic_vector(c_FTW_size-1 downto 0);
  signal loc_nco_ctrl_reset_r    : std_logic;
  signal wrs_frame_counter       : unsigned(15 downto 0);
  signal wrs_frame_counter_r     : unsigned(15 downto 0);
  signal wrs_frame_valid         : std_logic;
  signal wrs_nco_ctrl_reset      : std_logic;
  signal wrs_nco_ctrl_reset_r    : std_logic;
  signal wrs_rx_reset_nco_pulse  : std_logic;  

  signal wb_rfframerxtx_in       : t_wishbone_master_in;
  signal wb_rfframerxtx_out      : t_wishbone_master_out;

  signal wb_rf1_rfnco_in         : t_wishbone_master_in;
  signal wb_rf1_rfnco_out        : t_wishbone_master_out;
  signal wb_rf2_rfnco_in         : t_wishbone_master_in;  
  signal wb_rf2_rfnco_out        : t_wishbone_master_out;
  

  signal wb_dds_fpg_in           : t_wishbone_master_in;
  signal wb_dds_fpg_out          : t_wishbone_master_out;
  signal rf_reset_nco_pulse      : std_logic;

   -- ILA debug
  signal ila_clk                 : std_logic;
  signal ila_sigs                : std_logic_vector(127 downto 0);
  signal dbg : std_logic_vector(15 downto 0);

  signal spi_flash_cs_n_sys : std_logic;
  signal spi_flash_mosi_sys : std_logic;
  signal spi_flash_sck_sys : std_logic;
  signal spi_flash_cs_n_wrc : std_logic;
  signal spi_flash_mosi_wrc : std_logic;
  signal spi_flash_sck_wrc : std_logic;
  signal spi_flash_sck : std_logic;
  
begin

    --  Poweron reset.
    process (clk_sys_62m5)
    begin
        if rising_edge(clk_sys_62m5) then
            if poweron_counter /= 0 then
                poweron_counter <= poweron_counter - 1;
                rst_sys_n_r     <= '0';
                rst_sys_r       <= '1';
            else
                rst_sys_n_r     <= '1';
                rst_sys_r       <= '0';
            end if;
        end if;
    end process;

    rst_sys_n  <= rst_sys_n_r;
    rst_sys    <= rst_sys_r;

    ----------------------------------------------------------------------------
    -- Clock distribution/PLL and reset
    ----------------------------------------------------------------------------

    inst_IBUFDS_dmtd : IBUFGDS
        port map (
            O  => clk_dmtd_62m5,       -- Buffer output
            I  => clk_dmtd_62m5_p_i,   -- Diff_p buffer input
            IB => clk_dmtd_62m5_n_i ); -- Diff_n buffer input

    inst_IBUFDS_sys : IBUFGDS
        port map (
            O  => clk_sys_62m5_in,
            I  => clk_sys_62m5_p_i,
            IB => clk_sys_62m5_n_i );

    inst_BUFGMUX_CTRL : BUFGMUX_CTRL
        port map (
            O  => clk_sys_62m5,        -- 1-bit output: Clock output
            I0 => clk_dmtd_62m5,       -- 1-bit input: Clock input (S=0)
            I1 => clk62m5,             -- 1-bit input: Clock input (S=1)
            S  => clk_sys_select );    -- 1-bit input: Clock select
    
    inst_IBUFDS_clk_ext_10m : IBUFDS
        port map (
            O  => clk_ext_10m_in,      -- Buffer output
            I  => clk_ext_10m_p_i,     -- Diff_p buffer input
            IB => clk_ext_10m_n_i );   -- Diff_n buffer input

    wr2rf_sysclks_i : entity work.wr2rf_sysclks
        port map (
            clk_sys_i       => clk_sys_62m5_in,
            reset_i         => clk_sys_reset,

            mmcm_locked_o       => mmcm_locked,
            mmcm_shift_incdec_i => mmcm_shift_incdec,
            mmcm_shift_en_i     => mmcm_shift_en,
            mmcm_shift_busy_o   => mmcm_shift_busy,

            clk62m5_o       => clk62m5,
            rst_clk62m5_n_o => rst_clk62m5_n,
            rst_clk62m5_o   => rst_clk62m5,

            clk10m_o        => clk_ext_10m_o,

            clk125m_o       => clk125m,
            rst_clk125m_n_o => rst_clk125m_n,
            rst_clk125m_o   => rst_clk125m,

            clk250m_o       => clk250m,
            rst_clk250m_n_o => rst_clk250m_n,
            rst_clk250m_o   => rst_clk250m,

            clk200m_o       => clk200m,
            rst_clk200m_n_o => rst_clk200m_n,
            rst_clk200m_o   => rst_clk200m );


    ------------------------------------------------------------------------------
    -- Dedicated clock for GTP
    ------------------------------------------------------------------------------

    inst_gtx_clk0_buf : IBUFDS_GTE2
        generic map(
            CLKCM_CFG    => true,
            CLKRCV_TRST  => true,
            CLKSWING_CFG => "11" )
        port map(
            O     => clk_gtx_125m,
            ODIV2 => open,
            CEB   => '0',
            I     => clk_125m_gtx_p_i,
            IB    => clk_125m_gtx_n_i );

    inst_GTX_QPLL : entity work.wr_gtx_phy_kintex7_lp_qpll
        generic map (
            g_simulation => g_simulation )
        port map (
            rst_i          => gtx_qpll_reset,
            clk_gtx_i      => clk_gtx_125m,
            clk_sys_i      => clk_sys_62m5,
            locked_o       => gtx_qpll_locked,
            qpll_clk_o     => gtx_qpll_clk,
            qpll_ref_clk_o => gtx_qpll_ref_clk );

    inst_GTX_Link0 : entity work.wr_gtx_phy_kintex7_lp
        generic map(
            g_simulation => g_simulation)
        port map(
            qpll_clk_i         => gtx_qpll_clk,
            qpll_ref_clk_i     => gtx_qpll_ref_clk,
            qpll_locked_i      => gtx_qpll_locked,
            qpll_reset_o       => gtx_qpll_reset,
            clk_dmtd_i         => clk_dmtd_62m5,
            clk_ref_i          => clk_sys_62m5,
            tx_data_i          => phy_tx_data,
            tx_k_i             => phy_tx_k,
            tx_disparity_o     => phy_tx_disparity,
            tx_enc_err_o       => phy_tx_enc_err,
            rx_rbclk_o         => phy_rx_rbclk,
            rx_data_o          => phy_rx_data,
            rx_k_o             => phy_rx_k,
            rx_enc_err_o       => phy_rx_enc_err,
--      rx_bitslide_o  => phy_rx_bitslide,
            rx_rbclk_sampled_o => phy_rx_rbclk_sampled,
            rst_i              => phy_rst,
            debug_i            => phy_lpc_ctrl,
            debug_o            => phy_lpc_stat,
            loopen_i           => phy_loopen,
            pad_txn_o          => sfp1_tx_n_o,
            pad_txp_o          => sfp1_tx_p_o,
            pad_rxn_i          => sfp1_rx_n_i,
            pad_rxp_i          => sfp1_rx_p_i,
            tx_clk_o           => open,
            tx_locked_o        => open,
            tx_prbs_sel_i      => "000",
            rdy_o              => phy_rdy );

    inst_WR_CORE : entity work.xwr_core
        generic map(
            g_simulation                     => g_simulation,
            g_with_external_clock_input      => true,
            --
            g_board_name                     => "WRRF",
            g_ram_address_space_size_kb      => 256,
            g_phys_uart                      => true,
            g_virtual_uart                   => true,
            g_aux_clks                       => 0,
            g_ep_rxbuf_size                  => 1024,
            g_tx_runt_padding                => true,
            g_pcs_16bit                      => true,
            g_dpram_initf                    => g_dpram_initf,
            g_dpram_size                     => (131072+65536)/4,
            g_interface_mode                 => PIPELINED,
            g_address_granularity            => BYTE,
            g_aux_sdb                        => c_wrc_periph3_sdb,
            g_softpll_enable_debugger        => true,
            g_softpll_use_sampled_ref_clocks => true,
            g_vuart_fifo_size                => 1024 )
        port map(
            clk_sys_i              => clk_sys_62m5,
            clk_dmtd_i             => clk_dmtd_62m5,
            clk_ref_i              => clk_sys_62m5,  --clk_125m_ref,
            clk_ext_mul_i          => '0',
            clk_ext_mul_locked_i   => '1',
            clk_ext_i              => clk_ext_10m_in,
            pps_ext_i              => pps_i,
            rst_n_i                => wrcore_reset_n,
            dac_hpll_load_p1_o     => dac_hpll_load_p1_o,
            dac_hpll_data_o        => dac_hpll_data_o,
            dac_dpll_load_p1_o     => dac_dpll_load_p1_o,
            dac_dpll_data_o        => dac_dpll_data_o,
            phy_ref_clk_i          => clk_sys_62m5,
            phy_tx_data_o          => phy_tx_data,
            phy_tx_k_o             => phy_tx_k,
            phy_tx_disparity_i     => phy_tx_disparity,
            phy_tx_enc_err_i       => phy_tx_enc_err,
            phy_rx_data_i          => phy_rx_data,
            phy_rx_rbclk_i         => phy_rx_rbclk,
            phy_rx_k_i             => phy_rx_k,
            phy_rx_enc_err_i       => phy_rx_enc_err,
            phy_rx_bitslide_i      => "00000",  --phy_rx_bitslide,
            phy_rst_o              => phy_rst,
            phy_rdy_i              => phy_rdy,
            phy_lpc_ctrl_o         => phy_lpc_ctrl,
            phy_lpc_stat_i         => phy_lpc_stat,
            phy_rx_rbclk_sampled_i => phy_rx_rbclk_sampled,
            phy_loopen_o           => phy_loopen,
            phy_loopen_vec_o       => phy_loopen_vec,
            phy_tx_prbs_sel_o      => phy_prbs_sel,
            phy_sfp_tx_fault_i     => sfp1_tx_fault_i,
            phy_sfp_los_i          => sfp1_los_i,
            phy_sfp_tx_disable_o   => sfp1_tx_disable_o,
            led_act_o              => sfp1_led_active_o,
            led_link_o             => sfp1_led_link_o,
            scl_o                  => wr1_scl_out,
            scl_i                  => wr1_scl_in,
            sda_o                  => wr1_sda_out,
            sda_i                  => wr1_sda_in,
            sfp_scl_o              => sfp1_scl_out,
            sfp_scl_i              => sfp1_scl_in,
            sfp_sda_o              => sfp1_sda_out,
            sfp_sda_i              => sfp1_sda_in,
            sfp_det_i              => sfp1_detect_i,
            spi_sclk_o             => spi_flash_sck_wrc,
            spi_ncs_o              => spi_flash_cs_n_wrc,
            spi_mosi_o             => spi_flash_mosi_wrc,
            spi_miso_i             => spi_flash_miso_i,
            uart_rxd_i             => '1',
            uart_txd_o             => open,
            owr_pwren_o            => open,
            owr_en_o               => owr_en,
            owr_i                  => owr_i,

            slave_i                => wb_wrc_in,
            slave_o                => wb_wrc_out,
            aux_master_o           => wb_aux_out,
            aux_master_i           => wb_aux_in,

            wrf_src_o              => wrf_src_out,
            wrf_src_i              => wrf_src_in,
            wrf_snk_o              => wrf_snk_out,
            wrf_snk_i              => wrf_snk_in,
            timestamps_o           => open,
            timestamps_ack_i       => '1',
            fc_tx_pause_req_i      => '0',
            fc_tx_pause_delay_i    => x"0000",
            fc_tx_pause_ready_o    => open,
            tm_link_up_o           => tm_link_up,
            tm_dac_value_o         => open,
            tm_dac_wr_o            => open,
            tm_clk_aux_lock_en_i   => (others => '0'),
            tm_clk_aux_locked_o    => open,
            tm_time_valid_o        => tm_time_valid,
            tm_tai_o               => tm_tai,
            tm_cycles_o            => tm_cycles,
            pps_p_o                => pps_p,
            pps_led_o              => open,
            pps_csync_o            => pps_csync,
            rst_aux_n_o            => open,
            link_ok_o              => open );

    inst_xwr_streamers : entity work.xwr_streamers
       generic map (
          g_streamers_op_mode    => g_streamers_op_mode,
          g_clk_ref_rate         => 62500000,
          g_tx_streamer_params   => g_tx_streamer_params,
          g_rx_streamer_params   => g_rx_streamer_params,
          g_simulation           => g_simulation )
       port map (
          clk_sys_i              => clk_sys_62m5,
          clk_ref_i              => clk_sys_62m5,

          rst_n_i                => rst_sys_n,
          src_i                  => wrf_snk_out,
          src_o                  => wrf_snk_in,
          snk_i                  => wrf_src_out,
          snk_o                  => wrf_src_in,

          -- REF clock domain
          tx_data_i              => wrs_tx_data,
          tx_valid_i             => wrs_tx_valid,
          tx_dreq_o              => wrs_tx_dreq,
          tx_last_p1_i           => wrs_tx_last,
          tx_flush_p1_i          => wrs_tx_flush,
          rx_first_p1_o          => wrs_rx_first,
          rx_last_p1_o           => wrs_rx_last,
          rx_data_o              => wrs_rx_data,
          rx_valid_o             => wrs_rx_valid,
          rx_dreq_i              => wrs_rx_dreq,

          -- WR timing depends on clk_dsp_62_5 being IN PHASE with the WR ref clock
          -- ensure this by proper clock/PLL routing!!!!
          tm_time_valid_i        => tm_time_valid,
          tm_tai_i               => tm_tai,
          tm_cycles_i            => tm_cycles,
          link_ok_i              => link_ok,
          wb_slave_i             => wb_aux_out,
          wb_slave_o             => wb_aux_in,
          snmp_array_o           => aux_diag_in(c_WR_STREAMERS_ARR_SIZE_OUT-1 downto 0),
          snmp_array_i           => aux_diag_out(c_WR_STREAMERS_ARR_SIZE_IN-1 downto 0),
          tx_streamer_cfg_i      => wrs_tx_cfg,
          rx_streamer_cfg_i      => wrs_rx_cfg );


    inst_RFFrameTransceiver : entity work.RFFrameTransceiver
        port map(
            clk_i                  => clk_sys_62m5,
            rst_n_i                => rst_sys_n,

            tx_data_o              => wrs_tx_data,
            tx_valid_o             => wrs_tx_valid,
            tx_dreq_i              => wrs_tx_dreq,
            tx_last_p1_o           => wrs_tx_last,
            tx_flush_p1_o          => wrs_tx_flush,
            rx_data_i              => wrs_rx_data,
            rx_valid_i             => wrs_rx_valid,
            rx_first_p1_i          => wrs_rx_first,
            rx_dreq_o              => wrs_rx_dreq,
            rx_last_p1_i           => wrs_rx_last,

            rx_RFFrameHeader_o     => wrs_rx_RFFrameHeader,
            rx_RFmFramePayloads_o  => wrs_rx_RFmFramePayloads,
            rx_RFsFramePayloads_o  => wrs_rx_RFsFramePayloads,
            rx_Frame_valid_pX_o    => wrs_rx_Frame_valid_pX,
            rx_Frame_typeID_o      => wrs_rx_Frame_typeID,

            ready_o                => wrs_tx_ready,
            tx_TransmitFrame_p1_i  => wrs_tx_TransmitFrame_p1,
            tx_RFFrameHeader_i     => wrs_tx_RFFrameHeader,
            tx_RFmFramePayloads_i  => wrs_tx_RFmFramePayloads,
            tx_RFsFramePayloads_i  => wrs_tx_RFsFramePayloads,

            wb_slave_i             => wb_rfframerxtx_out,
            wb_slave_o             => wb_rfframerxtx_in );

   process (clk_sys_62m5)
   begin
      if rising_edge (clk_sys_62m5) then
         if rst_sys_n = '0' then
            wrs_frame_counter_r  <= (others => '0');
            wrs_nco_ctrl_reset_r <= '1';
            loc_nco_h1_ftw_r     <= X"001687012c84";
            loc_nco_h1_prog_r    <= (others => '0');
            loc_nco_ctrl_reset_r <= '1';
         else
            wrs_frame_counter_r    <= wrs_frame_counter;
            wrs_nco_ctrl_reset_r   <= wrs_nco_ctrl_reset;            
            loc_nco_update_valid_r <= loc_nco_update_valid;
            if (loc_nco_update_valid = '1') then
               loc_nco_h1_ftw_r     <= loc_nco_h1_ftw;
               loc_nco_h1_prog_r    <= loc_nco_h1_prog;
               loc_nco_ctrl_reset_r <= loc_nco_ctrl_reset;
            end if;
         end if;
      end if;
   end process;


    wrs_frame_valid        <= '1' when rx_Frame_valid_pX              = '1' and rx_Frame_typeID = c_ID_RFmFrame else '0';
    wrs_rx_reset_nco_pulse <= '1' when rx_RFmFramePayloads.control(0) = '1' and wrs_frame_valid = '1'           else '0';
    wrs_frame_counter      <= wrs_frame_counter_r + 1 when wrs_frame_valid = '1' else wrs_frame_counter_r;        

    loc_rx_RFmFramePayloads.control(0)  <= loc_nco_ctrl_reset_r;
    loc_rx_RFmFramePayloads.FTW_H1_main <= loc_nco_h1_ftw_r;
    loc_rx_RFmFramePayloads.FTW_H1_prog <= loc_nco_h1_prog_r;
    loc_rx_Frame_valid_pX               <= loc_nco_update_valid;
    loc_rx_reset_nco_pulse              <= '1' when loc_nco_update_valid = '1' and loc_nco_update_valid_r = '0' else '0';

    rx_RFmFramePayloads <= loc_rx_RFmFramePayloads when loc_or_wrs_params_sel = '1' else wrs_rx_RFmFramePayloads;
    rx_Frame_valid_pX   <= loc_rx_Frame_valid_pX   when loc_or_wrs_params_sel = '1' else wrs_rx_Frame_valid_pX;
    rx_reset_nco_pulse  <= loc_rx_reset_nco_pulse  when loc_or_wrs_params_sel = '1' else wrs_rx_reset_nco_pulse;

    wr2rf_dds_i : entity work.wr2rf_dds
       generic map (
          g_use_fine_delay => false )
       port map (
          clk_sys_i        => clk_sys_62m5,
          clk_ref_i        => clk_sys_62m5,
          rst_sys_n_i      => rst_sys_n,

          pps_csync_i      => pps_csync,
          rf_reset_nco_i   => rx_reset_nco_pulse,

          dds_sync_p_o     => dds_sync_p_o,
          dds_sync_n_o     => dds_sync_n_o,

          wb_slave_i       => wb_dds_fpg_out,
          wb_slave_o       => wb_dds_fpg_in );

    wr2rf_rfchan_wrap_i : entity work.wr2rf_rfchan_wrap
        port map (
            clk125m_i             => clk125m,
            clk250m_i             => clk250m,
            rst_clk125m_i         => rst_clk125m,
            rst_clk125m_n_i       => rst_clk125m_n,            
            rst_clk250m_i         => rst_clk250m,

            rx_RFmFramePayloads_i => rx_RFmFramePayloads,
            rx_Frame_valid_pX_i   => rx_Frame_valid_pX,

            wb_rf1_rfnco_slave_i  => wb_rf1_rfnco_out,
            wb_rf1_rfnco_slave_o  => wb_rf1_rfnco_in,
            wb_rf2_rfnco_slave_i  => wb_rf2_rfnco_out,
            wb_rf2_rfnco_slave_o  => wb_rf2_rfnco_in,            

            rf1_sync_o            => rf1_sync,
            rf2_sync_o            => rf2_sync,

            rf1_iqdac_data_p_o    => rf1_iqdac_data_p_o,
            rf1_iqdac_data_n_o    => rf1_iqdac_data_n_o,
            rf1_iqdac_dci_p_o     => rf1_iqdac_dci_p_o,
            rf1_iqdac_dci_n_o     => rf1_iqdac_dci_n_o,

            rf2_iqdac_data_p_o    => rf2_iqdac_data_p_o,
            rf2_iqdac_data_n_o    => rf2_iqdac_data_n_o,
            rf2_iqdac_dci_p_o     => rf2_iqdac_dci_p_o,
            rf2_iqdac_dci_n_o     => rf2_iqdac_dci_n_o );

    inst_ocxo_DAC : entity work.gc_serial_dac
        generic map (
            g_num_data_bits  => 16,
            g_num_extra_bits => 8,
            g_num_cs_select  => 1,
            g_sclk_polarity  => 0)
        port map (
            clk_i         => clk_sys_62m5,
            rst_n_i       => rst_sys_n,
            value_i       => dac_dpll_data_o,
            cs_sel_i      => "1",
            load_i        => dac_dpll_load_p1_o,
            sclk_divsel_i => "010",
            dac_cs_n_o(0) => wr_dac_ocxo_cs_n_o,
            dac_sclk_o    => wr_dac_ocxo_sclk_o,
            dac_sdata_o   => wr_dac_ocxo_din_o);

    inst_dmtd_DAC : entity work.gc_serial_dac
        generic map (
            g_num_data_bits  => 16,
            g_num_extra_bits => 8,
            g_num_cs_select  => 1,
            g_sclk_polarity  => 0)
        port map (
            clk_i         => clk_sys_62m5,
            rst_n_i       => rst_sys_n,
            value_i       => dac_hpll_data_o,
            cs_sel_i      => "1",
            load_i        => dac_hpll_load_p1_o,
            sclk_divsel_i => "010",
            dac_cs_n_o(0) => wr_dac_dmtd_cs_n_o,
            dac_sclk_o    => wr_dac_dmtd_sclk_o,
            dac_sdata_o   => wr_dac_dmtd_din_o);

    sfp1_scl_b   <= '0' when sfp1_scl_out = '0' else 'Z';
    sfp1_sda_b   <= '0' when sfp1_sda_out = '0' else 'Z';
    sfp1_scl_in  <= sfp1_scl_b;
    sfp1_sda_in  <= sfp1_sda_b;

    sfp1_tx_disable_o <= '0';
    sfp1_rate_select_o <= '1';

    -- tie of sfp2 usused
    sfp2_tx_disable_o  <= '0';
    sfp2_led_active_o  <= '0';
    sfp2_led_link_o    <= '0';
    sfp2_rate_select_o <= '1';
    sfp2_scl_b         <= '0';
    sfp2_sda_b         <= '0';

    --  One wire: use port 0 ?
    wr_onewire_b <= '0' when (owr_en(0) = '1') else 'Z';
    owr_i(0)     <= wr_onewire_b;
    owr_i(1)     <= '1';

    wr_eeprom1_scl_b   <= '0' when wr1_scl_out = '0' else 'Z';
    wr_eeprom1_sda_b   <= '0' when wr1_sda_out = '0' else 'Z';
    wr1_scl_in  <= wr_eeprom1_scl_b;
    wr1_sda_in  <= wr_eeprom1_sda_b;

    wr_eeprom2_scl_b   <= 'Z';
    wr_eeprom2_sda_b   <= 'Z';

    process(clk_sys_62m5)
    begin
        if rising_edge(clk_sys_62m5) then
            pps_o <= pps_p;
        end if;
    end process;

    -----------------------------------------------------------------------------
    -- VME64x Core and buffers
    -----------------------------------------------------------------------------

    --  BERR and IRQ vme signals are inverted by the drivers. See schematics.
    vme_berr_o <= not vme_berr_n_o;
    vme_irq_o <= not vme_irq_n_o;

    inst_vme_core : entity work.xvme64x_core
        generic map (
            g_CLOCK_PERIOD    => 16,
            g_VME32           => False,
            g_ENABLE_CR_CSR   => False,
            g_USER_CSR_EXT    => False,
            g_WB_GRANULARITY  => BYTE,

            g_MANUFACTURER_ID => c_CERN_ID,
            g_BOARD_ID        => c_SVEC_ID,
            g_REVISION_ID     => c_SVEC_REVISION_ID,
            g_PROGRAM_ID      => c_SVEC_PROGRAM_ID,
            g_DECODER => (0      => (adem => x"fff00000", amcap => x"ee000000_00000000", dawpr => x"83"),
                          others => (adem => x"00000000", amcap => x"00000000_00000000", dawpr => x"83")) )
        port map (
            clk_i           => clk_sys_62m5,
            rst_n_i         => rst_sys_n,
            vme_i.as_n      => vme_as_n_i,
            vme_i.rst_n     => vme_sysreset_n_i,
            vme_i.write_n   => vme_write_n_i,
            vme_i.am        => vme_am_i,
            vme_i.ds_n      => vme_ds_n_i,
            vme_i.ga        => vme_ga,
            vme_i.lword_n   => vme_lword_n_b,
            vme_i.addr (23 downto 1)      => vme_addr_b (23 downto 1),
            vme_i.addr (31 downto 24)     => x"00",
            vme_i.data (15 downto 0)      => vme_data_b (15 downto 0),
            vme_i.data (31 downto 16) => x"0000",
            vme_i.iack_n    => vme_iack_n_i,
            vme_i.iackin_n  => vme_iackin_n_i,
            vme_o.berr_n    => vme_berr_n_o,
            vme_o.dtack_n   => vme_dtack_n_o,
            vme_o.retry_n   => vme_retry_n_o,
            vme_o.retry_oe  => vme_retry_oe_o,
            vme_o.lword_n   => vme_lword_n_b_out,
            vme_o.data (15 downto 0)      => vme_data_b_out,
            vme_o.data (31 downto 16)     => vme_data_b_out1,
            vme_o.addr      => vme_addr_b_out,
            vme_o.irq_n     => vme_irq_n_o,
            vme_o.iackout_n => vme_iackout_n_o,
            vme_o.dtack_oe  => vme_dtack_oe_o,
            vme_o.data_dir  => vme_data_dir_int,
            vme_o.data_oe_n => vme_data_oe_n_o,
            vme_o.addr_dir  => vme_addr_dir_o,
            vme_o.addr_oe_n => vme_addr_oe_n_o,
            wb_i   => wb_vme_in,
            wb_o   => wb_vme_out);

    vme_nogap <= not (vme_noga_n_i(0) xor vme_noga_n_i(1) xor vme_noga_n_i(2)
                      xor vme_noga_n_i(3) xor vme_noga_n_i(4));
    --  Use vme_ga_n_i if not unconnected (there are pull-up).
    --  Despite the name, the vme64x core want negated logic.
    vme_ga <= vme_gap_i & vme_ga_n_i when vme_ga_n_i /= (4 downto 0 => '1')
        else vme_nogap & vme_noga_n_i;

    -- VME tri-state buffers
    vme_data_b (15 downto 0)  <= vme_data_b_out    when vme_data_dir_int = '1'
                                 else (others => 'Z');

    vme_data_dir_o <= vme_data_dir_int;


    --  FIXME.
    rf1_iqdac_reset_o <= '0';
    rf2_iqdac_reset_o <= '0';

    inst_rf1_trigger : entity work.wr2rf_rftrigger
        port map (
            clk_sys_i         => clk_sys_62m5,
            rst_sys_n_i       => rst_sys_n,
            rst_sys_i         => rst_sys,
            clk_refclk_200m_i => clk200m,
            iodelay_reset_i   => iodelay_reset,
            wb_i              => wb_rf1_vtus_out,
            wb_o              => wb_rf1_vtus_in,

            rf_sync_i        => rf1_sync,
            rf_sync_p_o      => rf1_sync_p_o,
            rf_sync_n_o      => rf1_sync_n_o,

            rf_t1_clk_p_i    => rf1_t1_clk_p_i,
            rf_t1_clk_n_i    => rf1_t1_clk_n_i,
            rf_t1_p_o        => rf1_t1_p_o,
            rf_t1_n_o        => rf1_t1_n_o,
            rf_t1_rst_p_o    => rf1_t1_rst_p_o,
            rf_t1_rst_n_o    => rf1_t1_rst_n_o,
            rf_t1_start_i    => tmg_clk_i(1),
            rf_t1_stop_i     => tmg_io_i(1),

            rf_t2_clk_p_i    => rf1_t2_clk_p_i,
            rf_t2_clk_n_i    => rf1_t2_clk_n_i,
            rf_t2_p_o        => rf1_t2_p_o,
            rf_t2_n_o        => rf1_t2_n_o,
            rf_t2_rst_p_o    => rf1_t2_rst_p_o,
            rf_t2_rst_n_o    => rf1_t2_rst_n_o,
            rf_t2_start_i    => tmg_clk_i(1),
            rf_t2_stop_i     => tmg_io_i(1),

            rf_clk_bufg_o    => rf1_clk_bufg);

    inst_rf2_trigger : entity work.wr2rf_rftrigger
        port map (
            clk_sys_i         => clk_sys_62m5,
            rst_sys_n_i       => rst_sys_n,
            rst_sys_i         => rst_sys,
            clk_refclk_200m_i => clk200m,
            iodelay_reset_i   => iodelay_reset,
            wb_i              => wb_rf2_vtus_out,
            wb_o              => wb_rf2_vtus_in,

            rf_sync_i        => rf2_sync,
            rf_sync_p_o      => rf2_sync_p_o,
            rf_sync_n_o      => rf2_sync_n_o,

            rf_t1_clk_p_i    => rf2_t1_clk_p_i,
            rf_t1_clk_n_i    => rf2_t1_clk_n_i,
            rf_t1_p_o        => rf2_t1_p_o,
            rf_t1_n_o        => rf2_t1_n_o,
            rf_t1_rst_p_o    => rf2_t1_rst_p_o,
            rf_t1_rst_n_o    => rf2_t1_rst_n_o,
            rf_t1_start_i    => tmg_clk_i(2),
            rf_t1_stop_i     => tmg_io_i(2),

            rf_t2_clk_p_i    => rf2_t2_clk_p_i,
            rf_t2_clk_n_i    => rf2_t2_clk_n_i,
            rf_t2_p_o        => rf2_t2_p_o,
            rf_t2_n_o        => rf2_t2_n_o,
            rf_t2_rst_p_o    => rf2_t2_rst_p_o,
            rf_t2_rst_n_o    => rf2_t2_rst_n_o,
            rf_t2_start_i    => tmg_clk_i(2),
            rf_t2_stop_i     => tmg_io_i(2),

            rf_clk_bufg_o    => rf2_clk_bufg );


    wr2rf_vme_p0_i : entity work.wr2rf_vme_p0
        port map (
            rf1_clk_bufg_i   => rf1_clk_bufg,
            rf2_clk_bufg_i   => rf2_clk_bufg,
            vme_p0_h1_bsb_o  => vme_p0_h1_bsb_o,
            vme_p0_h1_c40_o  => vme_p0_h1_c40_o,
            vme_p0_h1_lbb_o  => vme_p0_h1_lbb_o,
            vme_p0_h1_hbb_o  => vme_p0_h1_hbb_o,
            vme_p0_h1_tc_o   => vme_p0_h1_tc_o,
            vme_p0_h2_bsb_o  => vme_p0_h2_bsb_o,
            vme_p0_h2_c40_o  => vme_p0_h2_c40_o,
            vme_p0_h2_lbb_o  => vme_p0_h2_lbb_o,
            vme_p0_h2_hbb_o  => vme_p0_h2_hbb_o,
            vme_p0_h2_tc_o   => vme_p0_h2_tc_o,
            vme_p0_oen_o     => vme_p0_oen_o );

    inst_wr2rf_regs_core: entity work.wr2rf_regs_core
       port map (
          clk_sys_i            => clk_sys_62m5,
          rst_sys_n_i          => rst_sys_n,

          clk_ref_i            => clk_sys_62m5,
          rst_ref_n_i          => rst_sys_n,

          wb_vme_i             => wb_vme_out,
          wb_vme_o             => wb_vme_in,

          wb_wrc_i             => wb_wrc_out,
          wb_wrc_o             => wb_wrc_in,

          wb_rfframerxtx_i     => wb_rfframerxtx_in,
          wb_rfframerxtx_o     => wb_rfframerxtx_out,

          wb_dds_fpg_i         => wb_dds_fpg_in,
          wb_dds_fpg_o         => wb_dds_fpg_out,

          wb_rf1_vtus_i        => wb_rf1_vtus_in,
          wb_rf1_vtus_o        => wb_rf1_vtus_out,
          wb_rf2_vtus_i        => wb_rf2_vtus_in,
          wb_rf2_vtus_o        => wb_rf2_vtus_out,

          wb_rf1_rfnco_i       => wb_rf1_rfnco_in,
          wb_rf1_rfnco_o       => wb_rf1_rfnco_out,
          wb_rf2_rfnco_i       => wb_rf2_rfnco_in,
          wb_rf2_rfnco_o       => wb_rf2_rfnco_out,
          
          mmcm_locked_i        => mmcm_locked,
          clk_sel_o            => clk_sys_select,
          clk_sys_reset_o      => clk_sys_reset,
          wrcore_reset_n_o     => wrcore_reset_n,
          iodelay_reset_o      => iodelay_reset,

          mmcm_shift_incdec_o  => mmcm_shift_incdec,
          mmcm_shift_en_o      => mmcm_shift_en,
          mmcm_shift_busy_i    => mmcm_shift_busy,

          tmg_clk_term_en_o    => tmg_clk_term_en_o,
          tmg_io_term_en_o     => tmg_io_term_en_o,
          tmg_clk_oe_o         => tmg_clk_oe_o,
          tmg_io_oe_o          => tmg_io_oe_o,
          ext_ref_dir_o        => ext_ref_dir_o,

          dds_cs_n_o           => dds_cs_n_o,
          rf1_iqdac_cs_n_o     => rf1_iqdac_cs_n_o,
          rf2_iqdac_cs_n_o     => rf2_iqdac_cs_n_o,
          rf_sclk_o            => rf_sclk_o,
          rf_sdi_o             => rf_sdi_o,
          rf_sdo_i             => rf_sdo_i,

          pll_main_cs_n_o      => pll_main_cs_n_o,
          pll_main_sclk_o      => pll_main_sclk_o,
          pll_main_sdi_o       => pll_main_sdi_o,
          pll_main_sdo_i       => pll_main_sdo_i,

          pll_main_error_i     => pll_main_stat_i(1),
          pll_main_locked_i    => pll_main_stat_i(2),
          pll_main_sync_o      => pll_main_sync_o,          

          dds_reset_o          => dds_reset_o,
          dds_ioupdate_o       => dds_ioupdate_o,
          dds_profile_o        => dds_profile_o,
          dds_sync_error_i     => dds_sync_error_i,

          ocxo_enabled_o       => ocxo_enabled,
          ocxo_uptime_i        => ocxo_uptime,

          wr2rfsd_nco_azimuthal_o  => open,
          wr2rfsd_nco_cabledelay_o => open,
          wr2rfsd_nco_hb_o         => open,
          loc_nco_ctrl_reset_o     => loc_nco_ctrl_reset,
          loc_nco_h1_ftw_o         => loc_nco_h1_ftw,
          loc_nco_h1_prog_o        => loc_nco_h1_prog,
          loc_nco_update_valid_o   => loc_nco_update_valid,
          loc_or_wrs_params_sel_o  => loc_or_wrs_params_sel,

          rf1_mux_sel_o        => rf1_mux_sel_o,
          rf1_mixer_en_o       => rf1_mixer_en_o,
          rf1_t1_delay_latch_o => rf1_t1_delay_latch_o,
          rf1_t1_delay_oen_o   => rf1_t1_delay_oen_o,
          rf1_t1_mux_sel_o     => rf1_t1_mux_sel_o,
          rf1_t2_delay_latch_o => rf1_t2_delay_latch_o,
          rf1_t2_delay_oen_o   => rf1_t2_delay_oen_o,
          rf1_t2_mux_sel_o     => rf1_t2_mux_sel_o,

          rf2_mux_sel_o        => rf2_mux_sel_o,
          rf2_mixer_en_o       => rf2_mixer_en_o,
          rf2_t1_delay_latch_o => rf2_t1_delay_latch_o,
          rf2_t1_delay_oen_o   => rf2_t1_delay_oen_o,
          rf2_t1_mux_sel_o     => rf2_t1_mux_sel_o,
          rf2_t2_delay_latch_o => rf2_t2_delay_latch_o,
          rf2_t2_delay_oen_o   => rf2_t2_delay_oen_o,
          rf2_t2_mux_sel_o     => rf2_t2_mux_sel_o,

          rf_delay_o           => rf_delay_o,

          flash_cs_n_o         => spi_flash_cs_n_sys,
          flash_miso_i         => spi_flash_miso_i,
          flash_mosi_o         => spi_flash_mosi_sys,
          flash_sck_o => spi_flash_sck_sys,

          tm_tai_i             => tm_tai,
          tm_cycles_i          => tm_cycles,
          tm_linkup_i          => tm_link_up,
          tm_time_valid_i      => tm_time_valid,

          pps_i                => pps );

    ocxo_en_o <= ocxo_enabled; 
    inst_wr2rf_ocxo : entity work.wr2rf_ocxo
       port map (
          clk_sys_i      => clk_sys_62m5,
          ocxo_enabled_i => ocxo_enabled,
          ocxo_uptime_o  => ocxo_uptime );    

    -- timing_io
    tmg_clk_oe_n_o  <= not tmg_clk_oe_o;
    tmg_io_oe_n_o   <= not tmg_io_oe_o;

    tmg_clk_o       <= "00";
    tmg_io_o        <= "0000";

    -- debug
    ila_sigs(0)  <= clk_sys_select;
    ila_sigs(1)  <= clk_sys_reset;
    ila_sigs(2)  <= mmcm_shift_incdec;
    ila_sigs(3)  <= mmcm_shift_en;
    ila_sigs(4)  <= mmcm_shift_busy;
    ila_sigs(5)  <= mmcm_locked;    
    ila_sigs(6)  <= wrcore_reset_n;
    ila_sigs(7)  <= iodelay_reset;

    
    ila_sigs(8)  <= wb_vme_out.stb;
    ila_sigs(9)  <= wb_vme_out.we;
    ila_sigs(10) <= wb_vme_out.cyc;
    ila_sigs(11) <= wb_vme_in.ack;
    ila_sigs(12) <= wb_vme_in.stall;

    ila_sigs(13) <= wb_wrc_in.stb;
    ila_sigs(14) <= wb_wrc_in.we;
    ila_sigs(15) <= wb_wrc_out.ack;
--    ila_sigs(31 downto 16)   <= wb_wrc_in.adr(15 downto 0);
    ila_sigs(31 downto 16)   <= phy_tx_data;
    ila_sigs(47 downto 32)   <= phy_rx_data;
    ila_sigs(63 downto 48)   <= dbg;
--    ila_sigs(63 downto 32)   <= wb_wrc_in.dat(31 downto 0);    
--    ila_sigs(95 downto 64)   <= wb_wrc_out.dat(31 downto 0);
    ila_sigs(95 downto 64)   <= (others => '0');
    ila_sigs(111 downto 96)  <= wb_wrc_in.adr(31 downto 16);
    ila_sigs(112)            <= gtx_qpll_reset;
    ila_sigs(113)            <= gtx_qpll_locked;
    ila_sigs(115 downto 114) <= phy_tx_k;
    ila_sigs(116)            <= phy_tx_enc_err;
    ila_sigs(117)            <= phy_rx_enc_err;
    ila_sigs(119 downto 118) <= phy_rx_k;    
    ila_sigs(120)            <= phy_rst;
    ila_sigs(121)            <= phy_rdy;
    ila_sigs(122)            <= sfp1_tx_fault_i;
    ila_sigs(123)            <= rx_Frame_valid_pX;
    ila_sigs(124)            <= loc_nco_ctrl_reset;
    ila_sigs(125)            <= loc_nco_update_valid;
    ila_sigs(126)            <= rf1_sync;
    ila_sigs(127)            <= rf2_sync;    

    ila_clk <= clk_sys_62m5;

    ila_i : entity work.ila
       generic map (
          ila_en      => true )
       port map (
          clk0        => ila_clk,
          ila0        => ila_sigs );


    spi_flash_cs_n_o <= spi_flash_cs_n_sys and spi_flash_cs_n_wrc;
    spi_flash_mosi_o <= spi_flash_mosi_sys or spi_flash_mosi_wrc;
    spi_flash_sck <= spi_flash_sck_sys or spi_flash_sck_wrc;
    
    STARTUPE2_inst : STARTUPE2
      generic map (
        PROG_USR => "FALSE",  -- Activate program event security feature. Requires encrypted bitstreams.
        SIM_CCLK_FREQ => 0.0  -- Set the Configuration Clock Frequency(ns) for simulation.
        )
      port map (
        CFGCLK => open,   -- 1-bit output: Configuration main clock output
        CFGMCLK => open,  -- 1-bit output: Configuration internal oscillator clock output
        EOS => open,      -- 1-bit output: Active high output signal indicating the End Of Startup.
        PREQ => open,     -- 1-bit output: PROGRAM request to fabric output
        CLK => '0',       -- 1-bit input: User start-up clock input
        GSR => '0',       -- 1-bit input: Global Set/Reset input (GSR cannot be used for the port name)
        GTS => '0',       -- 1-bit input: Global 3-state input (GTS cannot be used for the port name)
        KEYCLEARB => '0', -- 1-bit input: Clear AES Decrypter Key input from Battery-Backed RAM (BBRAM)
        PACK => '0',      -- 1-bit input: PROGRAM acknowledge input
        USRCCLKO => spi_flash_sck,   -- 1-bit input: User CCLK input
        USRCCLKTS => '0', -- 1-bit input: User CCLK 3-state enable input
        USRDONEO => '0',  -- 1-bit input: User DONE pin output control
        USRDONETS => '1'  -- 1-bit input: User DONE 3-state enable output
    );

    
end rtl;
