#include <stdio.h>
#include <stdlib.h>
#include "board.h"

#define AD9910_REF_FREQ ( 1000000000ULL ) // 1 GHz

struct ad9910_config_reg {
    int addr;
    uint64_t value;
    int nbits;
};

static struct ad9910_config_reg ad9910_default_config[] = {
//    {0, 0x0200 0002 | (1<<13), 32},              // CFR1, unidir mode for SDIO, autoclear phase accumulator on IOUPDATE
    {0, 0x02000002 | (1<<13), 32},              // CFR1, unidir mode for SDIO, autoclear phase accumulator on IOUPDATE
    {1, 0x00000800, 32},              // CFR2, TW - enabled sync pulse timing validation
    {2, 0x1f3fc000, 32},              // CFR3: no PLL
//    {3, 0x00007f64, 32},              // Aux DAC control: DAC Full scale current
    //{3, 0x11, 32}, // we take the default fsc setting
    {4, 0xffffffff, 32},              // IO update rate
    {7, 0x00000000, 32},              // default FTW
    {8, 0x0000, 16},                  // default phase offset
    {9, 0x00000000, 32},              // amplitude scale factors
    {0xa, 0x00000000, 32},            // multichip sync
    {0xe, 0x08b5000039374bc7ULL, 64}, // profile 0: 223.5 MHz, 0 deg phi, ASF 0.13
    {-1, 0, 0}};


#define AD9910_REG_CFR2 1
#define AD9910_DEFAULT_CFR2  0x400820

static void ad9910_probe(void)
{
    libwr2rf_dds_write(0, 0x0, 32); // unidir mode for SDIO
    libwr2rf_dds_write(0, 0x02000002, 32); // unidir mode for SDIO
    libwr2rf_dds_ioupdate();

    uint32_t cfr1 = libwr2rf_dds_read(0, 32);
    printf("AD9910: cfr1=0x%x \n", cfr1);

    uint32_t id = libwr2rf_dds_read(AD9910_REG_CFR2, 32);
    printf("AD9910: 0x%x (expected 0x%x)\n", id, AD9910_DEFAULT_CFR2);

    if (id != AD9910_DEFAULT_CFR2) {
      printf ("DDS not found\n");
      exit(1);
    }
}

static uint64_t ad9910_frequency_to_ftw(uint64_t freq_hz )
{
    // formula (2) from AD9910 datasheet, page 22
    return  (1ULL << 32) * freq_hz / AD9910_REF_FREQ;
}


static void ad9910_program(uint64_t ftw, int phase, int fs_current)
{
    int i;
    uint64_t prof0_cr = ftw | (0x8b5ULL << 48);

    printf("ad9910 FTW = %d\n", (uint32_t) ftw);

//    dev_dbg("ad9910_program [%08x%08x] asf %d!\n", (uint32_t)(prof0_cr >> 32), (uint32_t)prof0_cr, asf );

    for(i = 0; ad9910_default_config[i].addr >= 0; i++)
    {
        struct ad9910_config_reg r = ad9910_default_config[i];

        if(r.addr == 3)
        {
            r.value &= 0xffffff00;
            r.value |= fs_current;     // Aux DAC control: DAC Full scale current
  //          pp_printf("auxdac %08x\n", (uint32_t) r.value );
        } else
        if( r.addr == 0xe ) // profile 0
            r.value = prof0_cr;

        libwr2rf_dds_write(r.addr, r.value, r.nbits);
    }

    libwr2rf_dds_ioupdate();
}

#define AD9910_SYNC_VERIF_DELAY_TAPS 0

static void ad9910_configure_sync(int enable, int fine_delay_taps)
{
    uint32_t r10 = ((fine_delay_taps & 0x1f) << 3)
                    | (enable ? ( 1<<27) : 0)
                    | (AD9910_SYNC_VERIF_DELAY_TAPS << 28) | (1<<26);

    libwr2rf_dds_write(0xa, 0, 32);
    libwr2rf_dds_write(0x1, 0x00000820, 32);      // CFR2, TW - enabled sync pulse timing validation
    libwr2rf_dds_ioupdate();

    libwr2rf_dds_write(0x1, 0x00000800, 32);              // CFR2, TW - enabled sync pulse timing validation
    libwr2rf_dds_write(0xa, r10 , 32);
    libwr2rf_dds_ioupdate();
}

void
libwr2rf_dds_init(void)
{
  /* Probe.  */
  ad9910_probe();

  /* Configure sync.  */
  /* TODO.  */
  ad9910_configure_sync(0, 0);

  /* Program.  */
  uint64_t ftw = ad9910_frequency_to_ftw(223500000ULL);
  ad9910_program(ftw, 0, 0x0 );

}
