#include <stdio.h>
#include "board.h"
#include "regs.h"

struct dac_reg {
  uint8_t reg;
  uint8_t val;
};

static struct dac_reg dac_config[] =
  {
   /* 4 wires, MSB first, no reset.  */
   { DAC_REG_SPI_CONTROL, 0},

   /* 2 complement input data.  */
   { DAC_REG_DATA_CONTROL, 0},

   /* No power down.  */
   { DAC_REG_POWER_DOWN, 0},

   /* Init value.  */
   { DAC_REG_SETUP_HOLD, 0},
   { DAC_REG_TIMING_ADJ, 0},

   /* Normal mode.  */
   { DAC_REG_MIX_MODE, 0},

   /* DAC1 full scale adjustment: 20.0mA.  */
   { DAC_REG_DAC1_FSC_HI, 0x02},
   { DAC_REG_DAC1_FSC_LO, 0x00},

   /* AUXDAC1: 0.0mA, AUX1P active, source current  */
   { DAC_REG_AUXDAC1_HI, 0x00},
   { DAC_REG_AUXDAC1_LO, 0x00},

   /* DAC2 full scale adjustment: 20.0mA.  */
   { DAC_REG_DAC2_FSC_HI, 0x02},
   { DAC_REG_DAC2_FSC_LO, 0x00},

   /* AUXDAC2: 0.0mA, AUX2P active, source current  */
   { DAC_REG_AUXDAC2_HI, 0x00},
   { DAC_REG_AUXDAC2_LO, 0x00},
};

void libwr2rf_dac_init(unsigned dev)
{
  unsigned i;

  /* Assume SPI is already initialized.  */

  printf ("dac %u version: 0x%02x\n",
          dev, libwr2rf_dac_read(dev, DAC_REG_VERSION));

  for (i = 0; i < sizeof(dac_config)/sizeof(dac_config[0]); i++)
    libwr2rf_dac_write(dev, dac_config[i].reg, dac_config[i].val);
}

void libwr2rf_dac_set_setup_hold(unsigned dev, unsigned set, unsigned hld)
{
  libwr2rf_dac_write(dev, DAC_REG_SETUP_HOLD, (set << 4) | hld);
}

void libwr2rf_dac_set_timing_delay(unsigned dev, unsigned v)
{
  libwr2rf_dac_write(dev, DAC_REG_TIMING_ADJ, v);
}

unsigned libwr2rf_dac_get_seek(unsigned dev)
{
  return libwr2rf_dac_read(dev, DAC_REG_SEEK) & 1;
}

int libwr2rf_dac_port_timing(unsigned dev)
{
  struct port_res {
    unsigned seek;
    unsigned set;
    unsigned hld;
  };
  struct port_res results[32];
  unsigned seek0;
  unsigned seek1;
  unsigned hld;
  unsigned set;
  unsigned i;
  unsigned smp;
  unsigned lo;
  unsigned hi;

  /* See AD9783 data sheet rev C, p 26.  */

  /* Building the Array.  */

  for (smp = 0; smp < 32; smp++)
    {
      /* 1.  Set the values of SMP, SET and HLD to 0.  Read and record
             the value of the SEEK bit.  */

      /* 4.  Set the value of SET and HLD to 0.  Increment the value of SMP
             and record the value of the SEEK bit.  */

      libwr2rf_dac_set_setup_hold(dev, 0, 0);
      libwr2rf_dac_set_timing_delay(dev, smp);
      seek0 = libwr2rf_dac_get_seek(dev);

      /* 2.  With SMP and SET set to 0, increment the HLD value until
             the seek bit toggles, and then record the HLD value.  This
             measures the hold time as shown in figure 57.  */

      /* 5a.  Increment the HLD until the SEEK bit tooles, and then record
         the HLD value.  */
      hld = 15;
      for (i = 0; i < 16; i++) {
        libwr2rf_dac_set_setup_hold(dev, 0, i);
        seek1 = libwr2rf_dac_get_seek(dev);
        if (seek1 != seek0) {
          hld = i;
          break;
        }
      }

      /* 3.  With SMP and HLD set to 0, increment the SET value until
             the SEEK bit toggles, and then record the SET value.  This
             measures the setup time as shown in figure 57.  */

      /* 5b.  Set HLD to 0 and increment SET until the SEEK bit
              toggles, and then record the SET value.  */
      set = 15;
      for (i = 0; i < 16; i++) {
        libwr2rf_dac_set_setup_hold(dev, i, 0);
        seek1 = libwr2rf_dac_get_seek(dev);
        if (seek1 != seek0) {
          set = i;
          break;
        }
      }

      printf("dac %u: smp=%2u set=%2u hld=%2u\n", dev, smp, set, hld);

      results[smp].seek = seek0;
      results[smp].set = set;
      results[smp].hld = hld;
    }

  /* Determining the SMP value.  */

  /* 1.  Look for the SMP value that corresponds to the 0-to-1 transition
         of the SEEK bit in the table.  */
  for (lo = 2; lo < 32; lo++)
    if (results[lo - 1].seek == 0 && results[lo].seek == 1)
      break;
  if (lo == 32) {
    printf ("dac %u: no 0-to-1 transition\n", dev);
    return -1;
  }
  /* 2.  Look for the SMP value that corresponds to the 1-to-0 transition
         of the SEEK bit in the table.  */
  for (hi = lo + 1; hi < 32; hi++)
    if (results[hi - 1].seek == 1 && results[hi].seek == 0)
      break;
  if (hi == 32) {
    printf ("dac %u: no 1-to-0 transition\n", dev);
    return -1;
  }
  /* 3.  The same two values found in Step 1 and Step 2 indicate the
         valid sampling window.  */
  printf ("dac %u: sampling window: %u-%u\n", dev, lo, hi);

  /* 4.  The optimal SMP value in the valid sampling window is where
         the following two conditions are true: SET < HDL and |HLD - SET| is
         the smallest value.  */
  {
    unsigned best = 0xff;
    unsigned diff = 0xff;

    for (i = lo; i < hi; i++)
      {
        if (results[i].set < results[i].hld)
          {
            unsigned d = results[i].hld - results[i].set;
            if (d < diff)
              best = i;
          }
      }
    if (best > 32) {
      printf ("dac %u: no best sampling window\n", dev);
      return -1;
    }
    printf ("adc %u: best sampling: %u\n", dev, best);
    libwr2rf_dac_set_timing_delay(dev, best);
    return 0;
  }
}
